from .api import api
from .const import *
from .library import library
from .exceptions import *


class OscilloscopeChannelTriggerHystereses(object):
    """"""

    def __init__(self, handle, ch):
        self._handle = handle
        self._ch = ch

    def __getitem__(self, index):
        try:
            value = api.ScpChTrGetHysteresis(self._handle, self._ch, index)
            library.check_last_status_raise_on_error()
            return value
        except InvalidIndexError:
            raise IndexError()
        except NotSupportedError:
            if api.ScpChHasTrigger(self._handle, self._ch) == BOOL8_TRUE:
                raise IndexError()
            else:
                raise

    def __setitem__(self, index, value):
        try:
            api.ScpChTrSetHysteresis(self._handle, self._ch, index, value)
            library.check_last_status_raise_on_error()
        except InvalidIndexError:
            raise IndexError()
        except NotSupportedError:
            if api.ScpChHasTrigger(self._handle, self._ch) == BOOL8_TRUE:
                raise IndexError()
            else:
                raise

    def __len__(self):
        return self.count

    def _get_count(self):
        return api.ScpChTrGetHysteresisCount(self._handle, self._ch)

    count = property(_get_count)



import sys
import os
import platform

is_windows = platform.system() == 'Windows'
is_linux = platform.system() == 'Linux'
is_32bit = (sys.maxsize <= 2**32)

del sys, os, platform

if is_windows:
    from ._windows import DLL_PATH
elif is_linux:
    from ._linux import DLL_PATH
else:
    raise ImportError("unsupported platform")



from ctypes import *
from .api import api
from .const import *
from .library import library
from .exceptions import InvalidIndexError
from .server import Server


class NetworkServers(object):
    """"""

    def __getitem__(self, index):
        try:
            return self.get_by_index(index)
        except (InvalidIndexError):
            raise IndexError()

    def __len__(self):
        return self.count

    def add(self, url):
        """ Add a server to the list of servers.

        :param url: XXX
        :returns: Instance of :class:`.Server`
        .. versionadded:: 0.9
        """
        url = url.encode('utf-8')
        handle = c_uint32(0)
        result = api.NetSrvAdd(url, STRING_LENGTH_NULL_TERMINATED, byref(handle))
        library.check_last_status_raise_on_error()
        return Server(handle.value) if result else None

    def remove(self, url, force):
        """ Remove a server from the list of servers

        :param url: Pointer to URL character buffer.
        :param force: If ``True`` all open devices are closed, if ``False`` remove only succeeds if no devices are open.
        :returns: ``True`` if removed successfully, ``False`` otherwise.
        .. versionadded:: 0.9
        """
        url = url.encode('utf-8')
        force = BOOL8_TRUE if force else BOOL8_FALSE
        result = api.NetSrvRemove(url, STRING_LENGTH_NULL_TERMINATED, force)
        library.check_last_status_raise_on_error()
        return result != BOOL8_FALSE

    def _get_count(self):
        """ Number of servers available. """
        value = api.NetSrvGetCount()
        library.check_last_status_raise_on_error()
        return value

    def get_by_index(self, index):
        """ Get the handle of a server, based on its index in the list of servers

        :param index: A server index, ``0`` .. NetSrvGetCount() - 1.
        :returns: Instance of :class:`.Server`.
        .. versionadded:: 0.9
        """
        result = api.NetSrvGetByIndex(index)
        library.check_last_status_raise_on_error()
        return Server(result)

    def get_by_url(self, url):
        """ Get the handle of a server, based on its URL

        :param url: Pointer to URL character buffer.
        :returns: Instance of :class:`.Server`.
        .. versionadded:: 0.9
        """
        url = url.encode('utf-8')
        result = api.NetSrvGetByURL(url, STRING_LENGTH_NULL_TERMINATED)
        library.check_last_status_raise_on_error()
        return Server(result)

    def set_callback_added(self, callback, data):
        """ Set a callback function which is called when a server is added to the server list.

        :param callback: A pointer to the callback function. Use ``None`` to disable.
        :param data: Optional user data.
        .. versionadded:: 0.9
        """
        api.NetSrvSetCallbackAdded(callback, data)
        library.check_last_status_raise_on_error()

    if platform.system() == 'Linux':
        def set_event_added(self, event):
            """ Set an event file descriptor which is set when a server is added to the server list.

            :param event: An event file descriptor. Use ``<0`` to disable.
            .. versionadded:: 0.9
            """
            api.NetSrvSetEventAdded(event)
            library.check_last_status_raise_on_error()

    if platform.system() == 'Windows':
        def set_event_added(self, event):
            """ Set an event object handle which is set when a server is added to the server list.

            :param event: A handle to the event object. Use ``None`` to disable.
            .. versionadded:: 0.9
            """
            api.NetSrvSetEventAdded(event)
            library.check_last_status_raise_on_error()

        def set_message_added(self, wnd):
            """ Set a window handle to which a #WM_LIBTIEPIE_NETSRV_ADDED message is sent when a server is added to the server list.

            :param wnd: A handle to the window whose window procedure is to receive the message. Use ``None`` to disable.
            .. versionadded:: 0.9
            """
            api.NetSrvSetMessageAdded(wnd)
            library.check_last_status_raise_on_error()

    count = property(_get_count)



from ctypes import create_string_buffer
from .api import api
from .const import *
from .utils import *
from .library import library
from .exceptions import InvalidValueError
from .oscilloscope import Oscilloscope
from .generator import Generator
from .i2chost import I2CHost
from .server import Server


class DeviceListItem(object):
    """"""

    def __init__(self, serial_number):
        self._serial_number = serial_number

    def open_device(self, device_type):
        """ Open a device .

        :param device_type: A device type.
        :returns: Instance of :class:`.Oscilloscope`, :class:`.Generator` or :class:`.I2CHost`.
        """
        result = api.LstOpenDevice(IDKIND_SERIALNUMBER, self._serial_number, device_type)
        library.check_last_status_raise_on_error()
        return library.create_object(result)

    def open_oscilloscope(self):
        """ Open an oscilloscope .

        :returns: Instance of :class:`.Oscilloscope`.
        """
        result = api.LstOpenOscilloscope(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return Oscilloscope(result)

    def open_generator(self):
        """ Open a generator .

        :returns: Instance of :class:`.Generator`.
        """
        result = api.LstOpenGenerator(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return Generator(result)

    def open_i2c_host(self):
        """ Open an I2C host .

        :returns: Instance of :class:`.I2CHost`.
        """
        result = api.LstOpenI2CHost(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return I2CHost(result)

    def can_open(self, device_type):
        """ Check whether the listed device can be opened.

        :param device_type: A device type.
        :returns: ``True`` if the device can be opened or ``False`` if not.
        """
        result = api.LstDevCanOpen(IDKIND_SERIALNUMBER, self._serial_number, device_type)
        library.check_last_status_raise_on_error()
        return result != BOOL8_FALSE

    def _get_product_id(self):
        """ Product id. """
        value = api.LstDevGetProductId(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return value

    def _get_vendor_id(self):
        """ Vendor id. """
        value = api.LstDevGetVendorId(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return value

    def _get_name(self):
        """ Full name. """
        length = api.LstDevGetName(IDKIND_SERIALNUMBER, self._serial_number, None, 0)
        library.check_last_status_raise_on_error()
        buf = create_string_buffer(length + 1)
        api.LstDevGetName(IDKIND_SERIALNUMBER, self._serial_number, buf, length)
        library.check_last_status_raise_on_error()
        return buf.value.decode('utf-8')

    def _get_name_short(self):
        """ Short name. """
        length = api.LstDevGetNameShort(IDKIND_SERIALNUMBER, self._serial_number, None, 0)
        library.check_last_status_raise_on_error()
        buf = create_string_buffer(length + 1)
        api.LstDevGetNameShort(IDKIND_SERIALNUMBER, self._serial_number, buf, length)
        library.check_last_status_raise_on_error()
        return buf.value.decode('utf-8')

    def _get_name_shortest(self):
        """ Short name wihout model postfix. """
        length = api.LstDevGetNameShortest(IDKIND_SERIALNUMBER, self._serial_number, None, 0)
        library.check_last_status_raise_on_error()
        buf = create_string_buffer(length + 1)
        api.LstDevGetNameShortest(IDKIND_SERIALNUMBER, self._serial_number, buf, length)
        library.check_last_status_raise_on_error()
        return buf.value.decode('utf-8')

    def _get_driver_version(self):
        """ Version number of the driver currently used. """
        value = api.LstDevGetDriverVersion(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return convert_version(value)

    def _get_recommended_driver_version(self):
        """ Version number of the recommended driver. """
        value = api.LstDevGetRecommendedDriverVersion(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return convert_version(value)

    def _get_firmware_version(self):
        """ Version number of the firmware currently used. """
        value = api.LstDevGetFirmwareVersion(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return convert_version(value)

    def _get_recommended_firmware_version(self):
        """ Version number of the recommended firmware. """
        value = api.LstDevGetRecommendedFirmwareVersion(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return convert_version(value)

    def _get_calibration_date(self):
        """ Calibration date. """
        value = api.LstDevGetCalibrationDate(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return convert_date(value)

    def _get_serial_number(self):
        """ Serial number. """
        value = api.LstDevGetSerialNumber(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return value

    def _get_ipv4_address(self):
        """ IPv4 address. """
        value = api.LstDevGetIPv4Address(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return value

    def _get_ip_port(self):
        """ IP port number. """
        value = api.LstDevGetIPPort(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return value

    def _get_has_server(self):
        """ Check whether the listed device is connected to a server. """
        value = api.LstDevHasServer(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_server(self):
        """ Server handle of the server the listed device is connected to. """
        value = api.LstDevGetServer(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return Server(value)

    def _get_types(self):
        """ Device types. """
        value = api.LstDevGetTypes(IDKIND_SERIALNUMBER, self._serial_number)
        library.check_last_status_raise_on_error()
        return value

    def _get_contained_serial_numbers(self):
        """ Serial numbers of the individual devices contained in a combined device. """
        count = api.LstDevGetContainedSerialNumbers(IDKIND_SERIALNUMBER, self._serial_number, None, 0)
        library.check_last_status_raise_on_error()
        values = (c_uint32 * count)()
        api.LstDevGetContainedSerialNumbers(IDKIND_SERIALNUMBER, self._serial_number, values, count)
        library.check_last_status_raise_on_error()
        return array('L', values)

    product_id = property(_get_product_id)
    vendor_id = property(_get_vendor_id)
    name = property(_get_name)
    name_short = property(_get_name_short)
    name_shortest = property(_get_name_shortest)
    driver_version = property(_get_driver_version)
    recommended_driver_version = property(_get_recommended_driver_version)
    firmware_version = property(_get_firmware_version)
    recommended_firmware_version = property(_get_recommended_firmware_version)
    calibration_date = property(_get_calibration_date)
    serial_number = property(_get_serial_number)
    ipv4_address = property(_get_ipv4_address)
    ip_port = property(_get_ip_port)
    has_server = property(_get_has_server)
    server = property(_get_server)
    types = property(_get_types)
    contained_serial_numbers = property(_get_contained_serial_numbers)



import sys
import os
import ctypes as ct

this_dir = os.path.dirname(os.path.abspath(__file__))
is_32bit = (sys.maxsize <= 2**32)
arch = 'x86' if is_32bit else 'x64'
arch_dir = os.path.join(this_dir, arch)

try:
    from ...__config__ import LIBTIEPIE as DLL_PATH
except ImportError:
    DLL_PATH = os.path.join(arch_dir, 'libtiepie.dll')



from array import array
from ctypes import c_void_p, c_double
from .api import api
from .const import *
from .library import library
from .device import Device


class Generator(Device):
    """"""

    def __init__(self, handle):
        super(Generator, self).__init__(handle)

    def _get_connector_type(self):
        """ Output connector type. """
        value = api.GenGetConnectorType(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_is_differential(self):
        """ Check whether the output is differential. """
        value = api.GenIsDifferential(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_impedance(self):
        """ Output impedance. """
        value = api.GenGetImpedance(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_resolution(self):
        """ DAC resolution. """
        value = api.GenGetResolution(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_output_value_min(self):
        """ Minimum output value. """
        value = api.GenGetOutputValueMin(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_output_value_max(self):
        """ Maximum output value. """
        value = api.GenGetOutputValueMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def get_output_value_min_max(self, min, max):
        """ Get the minimum and/or maximum output value.

        :param min: A pointer to a memory location for the minimum value, or ``None.``
        :param max: A pointer to a memory location for the maximum value, or ``None.``
        """
        api.GenGetOutputValueMinMax(self._handle, min, max)
        library.check_last_status_raise_on_error()

    def _get_is_controllable(self):
        """ Check whether a specified generator can be controlled. """
        value = api.GenIsControllable(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_is_running(self):
        """ Check whether the generator is running. """
        value = api.GenIsRunning(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_status(self):
        """ Current signal generation status """
        value = api.GenGetStatus(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_output_on(self):
        """ Check whether a specified generator is enabled """
        value = api.GenGetOutputOn(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _set_output_on(self, value):
        value = BOOL8_TRUE if value else BOOL8_FALSE
        api.GenSetOutputOn(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_has_output_invert(self):
        """ Check whether the output can be inverted """
        value = api.GenHasOutputInvert(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_output_invert(self):
        """ Check whether the output is inverted """
        value = api.GenGetOutputInvert(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _set_output_invert(self, value):
        value = BOOL8_TRUE if value else BOOL8_FALSE
        api.GenSetOutputInvert(self._handle, value)
        library.check_last_status_raise_on_error()

    def start(self):
        """ Start the signal generation.

        """
        api.GenStart(self._handle)
        library.check_last_status_raise_on_error()

    def stop(self):
        """ Stop the signal generation.

        """
        api.GenStop(self._handle)
        library.check_last_status_raise_on_error()

    def _get_signal_types(self):
        """ Supported signal types. """
        value = api.GenGetSignalTypes(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_signal_type(self):
        """ Currently selected signal type. """
        value = api.GenGetSignalType(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_signal_type(self, value):
        api.GenSetSignalType(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_has_amplitude(self):
        """ Check whether the current signal type supports controlling the signal amplitude. """
        value = api.GenHasAmplitude(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_amplitude_min(self):
        """ Minimum signal amplitude for the current signal type. """
        value = api.GenGetAmplitudeMin(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_amplitude_max(self):
        """ Maximum signal amplitude for the current signal type. """
        value = api.GenGetAmplitudeMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_amplitude(self):
        """ Currently set signal amplitude. """
        value = api.GenGetAmplitude(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_amplitude(self, value):
        api.GenSetAmplitude(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_amplitude(self, amplitude):
        """ Verify if a signal amplitude can be set, without actually setting the hardware itself.

        :param amplitude: The requested signal amplitude.
        :returns: The signal amplitude that would have been set, in Volt.
        """
        result = api.GenVerifyAmplitude(self._handle, amplitude)
        library.check_last_status_raise_on_error()
        return result

    def _get_amplitude_ranges(self):
        """ Supported amplitude ranges. """
        count = api.GenGetAmplitudeRanges(self._handle, None, 0)
        library.check_last_status_raise_on_error()
        values = (c_double * count)()
        api.GenGetAmplitudeRanges(self._handle, values, count)
        library.check_last_status_raise_on_error()
        return array('d', values)

    def _get_amplitude_range(self):
        """ Currently set amplitude range. """
        value = api.GenGetAmplitudeRange(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_amplitude_range(self, value):
        api.GenSetAmplitudeRange(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_amplitude_auto_ranging(self):
        """ Amplitude auto ranging setting. """
        value = api.GenGetAmplitudeAutoRanging(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _set_amplitude_auto_ranging(self, value):
        value = BOOL8_TRUE if value else BOOL8_FALSE
        api.GenSetAmplitudeAutoRanging(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_has_offset(self):
        """ Check whether the current signal type supports controlling the signal offset. """
        value = api.GenHasOffset(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_offset_min(self):
        """ Minimum offset for the current signal type. """
        value = api.GenGetOffsetMin(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_offset_max(self):
        """ Maximum offset for the current signal type. """
        value = api.GenGetOffsetMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_offset(self):
        """ Current signal offset. """
        value = api.GenGetOffset(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_offset(self, value):
        api.GenSetOffset(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_offset(self, offset):
        """ Verify if a signal offset can be set, without actually setting the hardware itself.

        :param offset: The requested signal offset, in Volt.
        :returns: The signal offset that would have been set, in Volt.
        """
        result = api.GenVerifyOffset(self._handle, offset)
        library.check_last_status_raise_on_error()
        return result

    def _get_frequency_modes(self):
        """ Supported generator frequency modes. """
        value = api.GenGetFrequencyModes(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_frequency_mode(self):
        """ Current generator frequency mode """
        value = api.GenGetFrequencyMode(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_frequency_mode(self, value):
        api.GenSetFrequencyMode(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_has_frequency(self):
        """ Check whether the current signal type and frequency mode support controlling the signal/sample frequency. """
        value = api.GenHasFrequency(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_frequency_min(self):
        """ Minimum signal/sample frequency with the current frequency mode. """
        value = api.GenGetFrequencyMin(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_frequency_max(self):
        """ Maximum signal/sample frequency with the current frequency mode and signal type. """
        value = api.GenGetFrequencyMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def get_frequency_min_max(self, frequency_mode, min, max):
        """ Get the minimum and maximum signal/sample frequency for a specified frequency mode and the current signal type.

        :param frequency_mode: The requested generator frequency mode, a FM_* value.
        :param min: A pointer to a memory location for the minimum frequency, or ``None.``
        :param max: A pointer to a memory location for the maximum frequency, or ``None.``
        """
        api.GenGetFrequencyMinMax(self._handle, frequency_mode, min, max)
        library.check_last_status_raise_on_error()

    def _get_frequency(self):
        """ Current signal/sample frequency. """
        value = api.GenGetFrequency(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_frequency(self, value):
        api.GenSetFrequency(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_frequency(self, frequency):
        """ Verify if a signal/sample frequency can be set, without actually setting the hardware itself.

        :param frequency: The requested signal/sample frequency, in Hz.
        :returns: The signal/sample frequency that would have been set, in Hz.
        """
        result = api.GenVerifyFrequency(self._handle, frequency)
        library.check_last_status_raise_on_error()
        return result

    def _get_has_phase(self):
        """ Check whether the specified generator and the current signal type support controlling the signal phase. """
        value = api.GenHasPhase(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_phase_min(self):
        """ Minimum signal phase. """
        value = api.GenGetPhaseMin(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_phase_max(self):
        """ Maximum signal phase. """
        value = api.GenGetPhaseMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_phase(self):
        """ Current signal phase. """
        value = api.GenGetPhase(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_phase(self, value):
        api.GenSetPhase(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_phase(self, phase):
        """ Verify if a phase can be set, without actually setting the hardware itself.

        :param phase: The requested signal phase, a number between ``0`` and ``1.``
        :returns: The signal phase that would have been set, a number between ``0`` and ``1.``
        """
        result = api.GenVerifyPhase(self._handle, phase)
        library.check_last_status_raise_on_error()
        return result

    def _get_has_symmetry(self):
        """ Check whether the current signal type supports controlling the signal symmetry. """
        value = api.GenHasSymmetry(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_symmetry_min(self):
        """ Minimum signal symmetry. """
        value = api.GenGetSymmetryMin(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_symmetry_max(self):
        """ Maximum signal symmetry. """
        value = api.GenGetSymmetryMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_symmetry(self):
        """ Current signal symmetry. """
        value = api.GenGetSymmetry(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_symmetry(self, value):
        api.GenSetSymmetry(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_symmetry(self, symmetry):
        """ Verify if a symmetry can be set, without actually setting the hardware itself.

        :param symmetry: The requested signal symmetry, a number between ``0`` and ``1.``
        :returns: The signal symmetry that would have been set, a number between ``0`` and ``1.``
        """
        result = api.GenVerifySymmetry(self._handle, symmetry)
        library.check_last_status_raise_on_error()
        return result

    def _get_has_width(self):
        """ Check whether the current signal type supports controlling the signal pulse width. """
        value = api.GenHasWidth(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_width_min(self):
        """ Minimum pulse width with the current signal frequency. """
        value = api.GenGetWidthMin(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_width_max(self):
        """ Maximum pulse width with the current signal frequency. """
        value = api.GenGetWidthMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_width(self):
        """ Current pulse width. """
        value = api.GenGetWidth(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_width(self, value):
        api.GenSetWidth(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_width(self, width):
        """ Verify if a pulse width can be set, without actually setting the hardware itself.

        :param width: The requested pulse width in seconds.
        :returns: The pulse width that would have been set, in seconds.
        """
        result = api.GenVerifyWidth(self._handle, width)
        library.check_last_status_raise_on_error()
        return result

    def _get_has_edge_time(self):
        """ Check whether the current signal type supports controlling the edge times. """
        value = api.GenHasEdgeTime(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_leading_edge_time_min(self):
        """ Minimum leading edge time with the current pulse width and signal frequency. """
        value = api.GenGetLeadingEdgeTimeMin(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_leading_edge_time_max(self):
        """ Maximum leading edge time with the current pulse width and signal frequency. """
        value = api.GenGetLeadingEdgeTimeMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_leading_edge_time(self):
        """ Current leading edge time with the current pulse width and signal frequency. """
        value = api.GenGetLeadingEdgeTime(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_leading_edge_time(self, value):
        api.GenSetLeadingEdgeTime(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_leading_edge_time(self, leading_edge_time):
        """ Verify if a leading edge time can be set for the current signal type and signal frequency, without actually setting the hardware itself.

        :param leading_edge_time: The requested leading edge time in seconds.
        :returns: The leading edge time that would have been set, in seconds.
        .. versionadded:: 0.6
        """
        result = api.GenVerifyLeadingEdgeTime(self._handle, leading_edge_time)
        library.check_last_status_raise_on_error()
        return result

    def _get_trailing_edge_time_min(self):
        """ Minimum trailing edge time with the current pulse width and signal frequency. """
        value = api.GenGetTrailingEdgeTimeMin(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_trailing_edge_time_max(self):
        """ Maximum trailing edge time with the current pulse width and signal frequency. """
        value = api.GenGetTrailingEdgeTimeMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_trailing_edge_time(self):
        """ Current trailing edge time with the current pulse width and signal frequency. """
        value = api.GenGetTrailingEdgeTime(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_trailing_edge_time(self, value):
        api.GenSetTrailingEdgeTime(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_trailing_edge_time(self, trailing_edge_time):
        """ Verify if a trailing edge time can be set for the current signal type and signal frequency, without actually setting the hardware itself.

        :param trailing_edge_time: The requested trailing edge time in seconds.
        :returns: The trailing edge time that would have been set, in seconds.
        .. versionadded:: 0.6
        """
        result = api.GenVerifyTrailingEdgeTime(self._handle, trailing_edge_time)
        library.check_last_status_raise_on_error()
        return result

    def _get_has_data(self):
        """ Check whether the current signal type supports controlling the Arbitrary waveform buffer. """
        value = api.GenHasData(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_data_length_min(self):
        """ Minimum length of the waveform buffer. """
        value = api.GenGetDataLengthMin(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_data_length_max(self):
        """ Maximum length of the waveform buffer. """
        value = api.GenGetDataLengthMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_data_length(self):
        """ Length of the currently loaded waveform pattern. """
        value = api.GenGetDataLength(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def verify_data_length(self, data_length):
        """ Verify if a specified length of the waveform buffer for the current signal type can be set, without actually setting the hardware itself.

        :param data_length: The requested waveform buffer length in samples.
        :returns: The waveform buffer length that would have been set, in samples.
        """
        result = api.GenVerifyDataLength(self._handle, data_length)
        library.check_last_status_raise_on_error()
        return result

    def set_data(self, data):
        """ Load a waveform pattern into the waveform buffer.

        :param data: :class:`array.array` of floats, the waveform data.
        """
        if isinstance(data, array) and data.typecode == 'f':
            data_ptr = c_void_p(data.buffer_info()[0])
            data_ptr.__ref = data
            api.GenSetData(self._handle, data_ptr, len(data))
            library.check_last_status_raise_on_error()
        else:
            raise Exception('Invalid data, must be array.array with typecode = `f`')

    def _get_data_raw_type(self):
        """ Raw data type. """
        value = api.GenGetDataRawType(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def get_data_raw_value_range(self, min, zero, max):
        """ Get raw data minimum, equal to zero and maximum values.

        :param min: Pointer to buffer for possible minimum raw data value, or ``None.``
        :param zero: Pointer to buffer for equal to zero raw data value, or ``None.``
        :param max: Pointer to buffer for possible maximum raw data value, or ``None.``
        """
        api.GenGetDataRawValueRange(self._handle, min, zero, max)
        library.check_last_status_raise_on_error()

    def _get_data_raw_value_min(self):
        """ Get maximum raw data value. """
        value = api.GenGetDataRawValueMin(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_data_raw_value_zero(self):
        """ Get raw data value that equals zero. """
        value = api.GenGetDataRawValueZero(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_data_raw_value_max(self):
        """ Get minimum raw data value. """
        value = api.GenGetDataRawValueMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def set_data_raw(self, buffer, sample_count):
        """ Load a waveform pattern into the waveform buffer.

        :param buffer: Pointer to buffer with waveform data.
        :param sample_count: Number of samples in buffer.
        """
        api.GenSetDataRaw(self._handle, buffer, sample_count)
        library.check_last_status_raise_on_error()

    def _get_modes(self):
        """ Supported generator modes for the current signal type and frequency mode. """
        value = api.GenGetModes(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_modes_native(self):
        """ :class:`array.array` of supported generator modes, regardless of the signal type and frequency mode. """
        value = api.GenGetModesNative(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_mode(self):
        """ Current generator mode. """
        value = api.GenGetMode(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_mode(self, value):
        api.GenSetMode(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_is_burst_active(self):
        """ Check whether a burst is active. """
        value = api.GenIsBurstActive(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_burst_count_min(self):
        """ Minimum burst count for the current generator mode. """
        value = api.GenGetBurstCountMin(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_burst_count_max(self):
        """ Maximum burst count for the current generator mode. """
        value = api.GenGetBurstCountMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_burst_count(self):
        """ Current burst count for the current generator mode. """
        value = api.GenGetBurstCount(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_burst_count(self, value):
        api.GenSetBurstCount(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_burst_sample_count_min(self):
        """ Minimum burst sample count for the current generator mode. """
        value = api.GenGetBurstSampleCountMin(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_burst_sample_count_max(self):
        """ Maximum burst sample count for the current generator mode. """
        value = api.GenGetBurstSampleCountMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_burst_sample_count(self):
        """ Current burst sample count for the current generator mode. """
        value = api.GenGetBurstSampleCount(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_burst_sample_count(self, value):
        api.GenSetBurstSampleCount(self._handle, value)
        library.check_last_status_raise_on_error()

    def _get_burst_segment_count_min(self):
        """ Minimum burst segment count for the current settings. """
        value = api.GenGetBurstSegmentCountMin(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_burst_segment_count_max(self):
        """ Maximum burst segment count for the current settings. """
        value = api.GenGetBurstSegmentCountMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_burst_segment_count(self):
        """ Current burst segment count. """
        value = api.GenGetBurstSegmentCount(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_burst_segment_count(self, value):
        api.GenSetBurstSegmentCount(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_burst_segment_count(self, burst_segment_count):
        """ Verify if a burst segment count can be set, without actually setting the hardware itself.

        :param burst_segment_count: The requested burst segment count.
        :returns: The burst segment count that would have been set.
        """
        result = api.GenVerifyBurstSegmentCount(self._handle, burst_segment_count)
        library.check_last_status_raise_on_error()
        return result

    def set_callback_burst_completed(self, callback, data):
        """ Set a callback function which is called when the generator burst is completed.

        :param callback: A pointer to the callback function. Use ``None`` to disable.
        :param data: Optional user data.
        """
        api.GenSetCallbackBurstCompleted(self._handle, callback, data)
        library.check_last_status_raise_on_error()

    if platform.system() == 'Linux':
        def set_event_burst_completed(self, event):
            """ Set an event file descriptor which is set when the generator burst is completed.

            :param event: An event file descriptor. Use ``<0`` to disable.
            """
            api.GenSetEventBurstCompleted(self._handle, event)
            library.check_last_status_raise_on_error()

    if platform.system() == 'Windows':
        def set_event_burst_completed(self, event):
            """ Set an event object handle which is set when the generator burst is completed.

            :param event: A handle to the event object. Use ``None`` to disable.
            """
            api.GenSetEventBurstCompleted(self._handle, event)
            library.check_last_status_raise_on_error()

        def set_message_burst_completed(self, wnd, wparam, lparam):
            """ Set a window handle to which a #WM_LIBTIEPIE_GEN_BURSTCOMPLETED message is sent when the generator burst is completed.

            :param wnd: A handle to the window whose window procedure is to receive the message. Use ``None`` to disable.
            :param wparam: Optional user value for the ``wParam`` parameter of the message.
            :param lparam: Optional user value for the ``lParam`` parameter of the message.
            """
            api.GenSetMessageBurstCompleted(self._handle, wnd, wparam, lparam)
            library.check_last_status_raise_on_error()

    def set_callback_controllable_changed(self, callback, data):
        """ Set a callback function which is called when the generator controllable property changes.

        :param callback: A pointer to the callback function. Use ``None`` to disable.
        :param data: Optional user data.
        """
        api.GenSetCallbackControllableChanged(self._handle, callback, data)
        library.check_last_status_raise_on_error()

    if platform.system() == 'Linux':
        def set_event_controllable_changed(self, event):
            """ Set an event file descriptor which is set when the generator controllable property changes.

            :param event: An event file descriptor. Use ``<0`` to disable.
            """
            api.GenSetEventControllableChanged(self._handle, event)
            library.check_last_status_raise_on_error()

    if platform.system() == 'Windows':
        def set_event_controllable_changed(self, event):
            """ Set event object handle which is set when the generator controllable property changes.

            :param event: A handle to the event object. Use ``None`` to disable.
            """
            api.GenSetEventControllableChanged(self._handle, event)
            library.check_last_status_raise_on_error()

        def set_message_controllable_changed(self, wnd, wparam, lparam):
            """ Set window handle to which a #WM_LIBTIEPIE_GEN_CONTROLLABLECHANGED message is sent when the generator controllable property changes.

            :param wnd: A handle to the window whose window procedure is to receive the message. Use ``None`` to disable.
            :param wparam: Optional user value for the ``wParam`` parameter of the message.
            :param lparam: Optional user value for the ``lParam`` parameter of the message.
            """
            api.GenSetMessageControllableChanged(self._handle, wnd, wparam, lparam)
            library.check_last_status_raise_on_error()

    connector_type = property(_get_connector_type)
    is_differential = property(_get_is_differential)
    impedance = property(_get_impedance)
    resolution = property(_get_resolution)
    output_value_min = property(_get_output_value_min)
    output_value_max = property(_get_output_value_max)
    is_controllable = property(_get_is_controllable)
    is_running = property(_get_is_running)
    status = property(_get_status)
    output_on = property(_get_output_on, _set_output_on)
    has_output_invert = property(_get_has_output_invert)
    output_invert = property(_get_output_invert, _set_output_invert)
    signal_types = property(_get_signal_types)
    signal_type = property(_get_signal_type, _set_signal_type)
    has_amplitude = property(_get_has_amplitude)
    amplitude_min = property(_get_amplitude_min)
    amplitude_max = property(_get_amplitude_max)
    amplitude = property(_get_amplitude, _set_amplitude)
    amplitude_ranges = property(_get_amplitude_ranges)
    amplitude_range = property(_get_amplitude_range, _set_amplitude_range)
    amplitude_auto_ranging = property(_get_amplitude_auto_ranging, _set_amplitude_auto_ranging)
    has_offset = property(_get_has_offset)
    offset_min = property(_get_offset_min)
    offset_max = property(_get_offset_max)
    offset = property(_get_offset, _set_offset)
    frequency_modes = property(_get_frequency_modes)
    frequency_mode = property(_get_frequency_mode, _set_frequency_mode)
    has_frequency = property(_get_has_frequency)
    frequency_min = property(_get_frequency_min)
    frequency_max = property(_get_frequency_max)
    frequency = property(_get_frequency, _set_frequency)
    has_phase = property(_get_has_phase)
    phase_min = property(_get_phase_min)
    phase_max = property(_get_phase_max)
    phase = property(_get_phase, _set_phase)
    has_symmetry = property(_get_has_symmetry)
    symmetry_min = property(_get_symmetry_min)
    symmetry_max = property(_get_symmetry_max)
    symmetry = property(_get_symmetry, _set_symmetry)
    has_width = property(_get_has_width)
    width_min = property(_get_width_min)
    width_max = property(_get_width_max)
    width = property(_get_width, _set_width)
    has_edge_time = property(_get_has_edge_time)
    leading_edge_time_min = property(_get_leading_edge_time_min)
    leading_edge_time_max = property(_get_leading_edge_time_max)
    leading_edge_time = property(_get_leading_edge_time, _set_leading_edge_time)
    trailing_edge_time_min = property(_get_trailing_edge_time_min)
    trailing_edge_time_max = property(_get_trailing_edge_time_max)
    trailing_edge_time = property(_get_trailing_edge_time, _set_trailing_edge_time)
    has_data = property(_get_has_data)
    data_length_min = property(_get_data_length_min)
    data_length_max = property(_get_data_length_max)
    data_length = property(_get_data_length)
    data_raw_type = property(_get_data_raw_type)
    data_raw_value_min = property(_get_data_raw_value_min)
    data_raw_value_zero = property(_get_data_raw_value_zero)
    data_raw_value_max = property(_get_data_raw_value_max)
    modes = property(_get_modes)
    modes_native = property(_get_modes_native)
    mode = property(_get_mode, _set_mode)
    is_burst_active = property(_get_is_burst_active)
    burst_count_min = property(_get_burst_count_min)
    burst_count_max = property(_get_burst_count_max)
    burst_count = property(_get_burst_count, _set_burst_count)
    burst_sample_count_min = property(_get_burst_sample_count_min)
    burst_sample_count_max = property(_get_burst_sample_count_max)
    burst_sample_count = property(_get_burst_sample_count, _set_burst_sample_count)
    burst_segment_count_min = property(_get_burst_segment_count_min)
    burst_segment_count_max = property(_get_burst_segment_count_max)
    burst_segment_count = property(_get_burst_segment_count, _set_burst_segment_count)



from .api import api
from .const import *
from .library import library


class Object(object):
    """"""

    def __init__(self, handle):
        self._handle = handle

    def __del__(self):
        api.ObjClose(self._handle)

    def _get_is_removed(self):
        """ Check whether an object is removed. """
        value = api.ObjIsRemoved(self._handle)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_interfaces(self):
        """ Check which interface are supported by the specified object. """
        value = api.ObjGetInterfaces(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def set_event_callback(self, callback, data):
        """ Set a callback function which is called when an event occurs.

        :param callback: A pointer to the callback function. Use ``None`` to disable.
        :param data: Optional user data.
        .. versionadded:: 0.6
        """
        api.ObjSetEventCallback(self._handle, callback, data)
        library.check_last_status_raise_on_error()

    def get_event(self, event, value):
        """ Get an event form the event queue.

        :param event: Pointer to store the event.
        :param value: Pointer to store the event value or ``None.``
        :returns: ``True`` if an event is available, ``False`` otherwise.
        .. versionadded:: 0.6
        """
        result = api.ObjGetEvent(self._handle, event, value)
        library.check_last_status_raise_on_error()
        return result != BOOL8_FALSE

    if platform.system() == 'Linux':
        def set_event_event(self, event):
            """ Set an event file descriptor which is set when an event occurs.

            :param event: An event file descriptor. Use <tt><0</tt> to disable.
            .. versionadded:: 0.6
            """
            api.ObjSetEventEvent(self._handle, event)
            library.check_last_status_raise_on_error()

    if platform.system() == 'Windows':
        def set_event_event(self, event):
            """ Set an event object handle which is set when an event occurs.

            :param event: A handle to the event object. Use ``None`` to disable.
            .. versionadded:: 0.6
            """
            api.ObjSetEventEvent(self._handle, event)
            library.check_last_status_raise_on_error()

        def set_event_window_handle(self, wnd):
            """ Set a window handle to which a #WM_LIBTIEPIE_EVENT message is sent when an event occurs.

            :param wnd: A handle to the window whose window procedure is to receive the message. Use ``None`` to disable.
            .. versionadded:: 0.6
            """
            api.ObjSetEventWindowHandle(self._handle, wnd)
            library.check_last_status_raise_on_error()

    is_removed = property(_get_is_removed)
    interfaces = property(_get_interfaces)



from array import array
from .api import api
from .utils import *
from .const import *
from .exceptions import *


class Library(object):
    """"""

    def __init__(self):
        self.__exit = api.LibExit  # Prevent garbage collector from freeing the exit function
        api.LibInit()

    def __del__(self):
        self.__exit()

    def _get_config_str(self):
        result = '0x'
        for v in self.config:
            result += '{0:02x}'.format(v)
        return result

    def check_last_status_raise_on_error(self):
        status = self.last_status
        if status >= STATUS_SUCCESS:
            return
        elif status == STATUS_UNSUCCESSFUL:
            raise UnsuccessfulError()
        elif status == STATUS_NOT_SUPPORTED:
            raise NotSupportedError()
        elif status == STATUS_INVALID_HANDLE:
            raise InvalidHandleError()
        elif status == STATUS_INVALID_VALUE:
            raise InvalidValueError()
        elif status == STATUS_INVALID_CHANNEL:
            raise InvalidChannelError()
        elif status == STATUS_INVALID_TRIGGER_SOURCE:
            raise InvalidTriggerSourceError()
        elif status == STATUS_INVALID_DEVICE_TYPE:
            raise InvalidDeviceTypeError()
        elif status == STATUS_INVALID_DEVICE_INDEX:
            raise InvalidDeviceIndexError()
        elif status == STATUS_INVALID_PRODUCT_ID:
            raise InvalidProductIdError()
        elif status == STATUS_INVALID_DEVICE_SERIALNUMBER:
            raise InvalidDeviceSerialNumberError()
        elif status == STATUS_OBJECT_GONE:
            raise ObjectGoneError()
        elif status == STATUS_INTERNAL_ADDRESS:
            raise InternalAddressError()
        elif status == STATUS_NOT_CONTROLLABLE:
            raise NotControllableError()
        elif status == STATUS_BIT_ERROR:
            raise BitError()
        elif status == STATUS_NO_ACKNOWLEDGE:
            raise NoAcknowledgeError()
        elif status == STATUS_INVALID_CONTAINED_DEVICE_SERIALNUMBER:
            raise InvalidContainedDeviceSerialNumberError()
        elif status == STATUS_INVALID_INPUT:
            raise InvalidInputError()
        elif status == STATUS_INVALID_OUTPUT:
            raise InvalidOutputError()
        elif status == STATUS_INVALID_DRIVER:
            raise InvalidDriverError()
        elif status == STATUS_NOT_AVAILABLE:
            raise NotAvailableError()
        elif status == STATUS_INVALID_FIRMWARE:
            raise InvalidFirmwareError()
        elif status == STATUS_INVALID_INDEX:
            raise InvalidIndexError()
        elif status == STATUS_INVALID_EEPROM:
            raise InvalidEepromError()
        elif status == STATUS_INITIALIZATION_FAILED:
            raise InitializationFailedError()
        elif status == STATUS_LIBRARY_NOT_INITIALIZED:
            raise LibraryNotInitializedError()
        elif status == STATUS_NO_TRIGGER_ENABLED:
            raise NoTriggerEnabledError()
        elif status == STATUS_SYNCHRONIZATION_FAILED:
            raise SynchronizationFailedError()
        elif status == STATUS_INVALID_HS56_COMBINED_DEVICE:
            raise InvalidHS56CombinedDeviceError()
        elif status == STATUS_MEASUREMENT_RUNNING:
            raise MeasurementRunningError()
        elif status == STATUS_INITIALIZATION_ERROR_10001:
            raise InitializationError10001()
        elif status == STATUS_INITIALIZATION_ERROR_10002:
            raise InitializationError10002()
        elif status == STATUS_INITIALIZATION_ERROR_10003:
            raise InitializationError10003()
        elif status == STATUS_INITIALIZATION_ERROR_10004:
            raise InitializationError10004()
        elif status == STATUS_INITIALIZATION_ERROR_10005:
            raise InitializationError10005()
        elif status == STATUS_INITIALIZATION_ERROR_10006:
            raise InitializationError10006()
        else:
            raise LibTiePieException(status, self.last_status_str)

    def create_object(self, handle):
        interfaces = api.ObjGetInterfaces(handle)
        self.check_last_status_raise_on_error()

        if interfaces == (INTERFACE_DEVICE | INTERFACE_OSCILLOSCOPE):
            from .oscilloscope import Oscilloscope
            return Oscilloscope(handle)
        elif interfaces == (INTERFACE_DEVICE | INTERFACE_GENERATOR):
            from .generator import Generator
            return Generator(handle)
        elif interfaces == (INTERFACE_DEVICE | INTERFACE_I2CHOST):
            from .i2chost import I2CHost
            return I2CHost(handle)
        elif interfaces == INTERFACE_DEVICE:
            from .device import Device
            return Device(handle)
        elif interfaces == INTERFACE_SERVER:
            from .server import Server
            return Server(handle)
        else:
            raise InvalidValueError()

    def _get_is_initialized(self):
        """ Check whether the library's internal resources are initialized. """
        value = api.LibIsInitialized()
        return value != BOOL8_FALSE

    def _get_version(self):
        """ Library version number. """
        value = api.LibGetVersion()
        return convert_version(value)

    def _get_version_extra(self):
        """ Library version postfix. """
        value = api.LibGetVersionExtra()
        return value.decode('utf-8')

    def _get_config(self):
        """ Library configuration number. """
        count = api.LibGetConfig(None, 0)
        values = (c_uint8 * count)()
        api.LibGetConfig(values, count)
        return array('B', values)

    def _get_last_status(self):
        """ Last status value. """
        return api.LibGetLastStatus()

    def _get_last_status_str(self):
        """ Last status value as text. """
        value = api.LibGetLastStatusStr()
        return value.decode('utf-8')

    is_initialized = property(_get_is_initialized)
    version = property(_get_version)
    version_extra = property(_get_version_extra)
    config = property(_get_config)
    last_status = property(_get_last_status)
    last_status_str = property(_get_last_status_str)
    config_str = property(_get_config_str)


library = Library()



from .objectlist import ObjectList
from .api import api
from .oscilloscopechannel import OscilloscopeChannel


class OscilloscopeChannels(ObjectList):
    """"""

    def __init__(self, handle):
        super(OscilloscopeChannels, self).__init__()
        self._items = [OscilloscopeChannel(handle, i)
                       for i in range(api.ScpGetChannelCount(handle))]



from .api import api
from .const import *
from .library import library
from .oscilloscopechanneltriggerlevels import OscilloscopeChannelTriggerLevels
from .oscilloscopechanneltriggerhystereses import OscilloscopeChannelTriggerHystereses
from .oscilloscopechanneltriggertimes import OscilloscopeChannelTriggerTimes


class OscilloscopeChannelTrigger(object):
    """"""

    def __init__(self, handle, ch):
        self._handle = handle
        self._ch = ch
        self._levels = OscilloscopeChannelTriggerLevels(handle, ch)
        self._hystereses = OscilloscopeChannelTriggerHystereses(handle, ch)
        self._times = OscilloscopeChannelTriggerTimes(handle, ch)

    def _get_levels(self):
        return self._levels

    def _get_hystereses(self):
        return self._hystereses

    def _get_times(self):
        return self._times

    def _get_is_available(self):
        """ Check whether the channel trigger is available, with the current oscilloscope settings. """
        value = api.ScpChTrIsAvailable(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_is_triggered(self):
        """ Check whether the channel trigger caused a trigger. """
        value = api.ScpChTrIsTriggered(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _get_enabled(self):
        """ Check whether channel trigger is enabled. """
        value = api.ScpChTrGetEnabled(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value != BOOL8_FALSE

    def _set_enabled(self, value):
        value = BOOL8_TRUE if value else BOOL8_FALSE
        api.ScpChTrSetEnabled(self._handle, self._ch, value)
        library.check_last_status_raise_on_error()

    def _get_kinds(self):
        """ Supported channel trigger kinds with the currently selected measure mode. """
        value = api.ScpChTrGetKinds(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _get_kind(self):
        """ Currently selected channel trigger kind. """
        value = api.ScpChTrGetKind(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _set_kind(self, value):
        api.ScpChTrSetKind(self._handle, self._ch, value)
        library.check_last_status_raise_on_error()

    def _get_level_modes(self):
        """ Supported trigger level modes. """
        value = api.ScpChTrGetLevelModes(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _get_level_mode(self):
        """ Current trigger level mode. """
        value = api.ScpChTrGetLevelMode(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _set_level_mode(self, value):
        api.ScpChTrSetLevelMode(self._handle, self._ch, value)
        library.check_last_status_raise_on_error()

    def _get_conditions(self):
        """ Supported trigger conditions with the currently selected trigger kind. """
        value = api.ScpChTrGetConditions(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _get_condition(self):
        """ Current selected trigger condition. """
        value = api.ScpChTrGetCondition(self._handle, self._ch)
        library.check_last_status_raise_on_error()
        return value

    def _set_condition(self, value):
        api.ScpChTrSetCondition(self._handle, self._ch, value)
        library.check_last_status_raise_on_error()

    is_available = property(_get_is_available)
    is_triggered = property(_get_is_triggered)
    enabled = property(_get_enabled, _set_enabled)
    kinds = property(_get_kinds)
    kind = property(_get_kind, _set_kind)
    level_modes = property(_get_level_modes)
    level_mode = property(_get_level_mode, _set_level_mode)
    conditions = property(_get_conditions)
    condition = property(_get_condition, _set_condition)
    levels = property(_get_levels)
    hystereses = property(_get_hystereses)
    times = property(_get_times)



class LibTiePieException(Exception):
    """The base class for all LibTiePie exceptions."""
    def __init__(self, status, message):
        super(LibTiePieException, self).__init__(message)
        self.status = status


class UnsuccessfulError(LibTiePieException):
    """"""

    def __init__(self):
        super(UnsuccessfulError, self).__init__(UnsuccessfulError, 'Unsuccessful')


class NotSupportedError(LibTiePieException):
    """"""

    def __init__(self):
        super(NotSupportedError, self).__init__(NotSupportedError, 'Not supported')


class InvalidHandleError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidHandleError, self).__init__(InvalidHandleError, 'Invalid handle')


class InvalidValueError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidValueError, self).__init__(InvalidValueError, 'Invalid value')


class InvalidChannelError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidChannelError, self).__init__(InvalidChannelError, 'Invalid channel')


class InvalidTriggerSourceError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidTriggerSourceError, self).__init__(InvalidTriggerSourceError, 'Invalid trigger source')


class InvalidDeviceTypeError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidDeviceTypeError, self).__init__(InvalidDeviceTypeError, 'Invalid device type')


class InvalidDeviceIndexError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidDeviceIndexError, self).__init__(InvalidDeviceIndexError, 'Invalid device index')


class InvalidProductIdError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidProductIdError, self).__init__(InvalidProductIdError, 'Invalid product id')


class InvalidDeviceSerialNumberError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidDeviceSerialNumberError, self).__init__(InvalidDeviceSerialNumberError, 'Invalid device serialnumber')


class ObjectGoneError(LibTiePieException):
    """"""

    def __init__(self):
        super(ObjectGoneError, self).__init__(ObjectGoneError, 'Object gone')


DeviceGoneError = ObjectGoneError


class InternalAddressError(LibTiePieException):
    """"""

    def __init__(self):
        super(InternalAddressError, self).__init__(InternalAddressError, 'Internal address')


class NotControllableError(LibTiePieException):
    """"""

    def __init__(self):
        super(NotControllableError, self).__init__(NotControllableError, 'Not controllable')


class BitError(LibTiePieException):
    """"""

    def __init__(self):
        super(BitError, self).__init__(BitError, 'Bit error')


class NoAcknowledgeError(LibTiePieException):
    """"""

    def __init__(self):
        super(NoAcknowledgeError, self).__init__(NoAcknowledgeError, 'No acknowledge')


class InvalidContainedDeviceSerialNumberError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidContainedDeviceSerialNumberError, self).__init__(InvalidContainedDeviceSerialNumberError, 'Invalid contained device serialnumber')


class InvalidInputError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidInputError, self).__init__(InvalidInputError, 'Invalid input')


class InvalidOutputError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidOutputError, self).__init__(InvalidOutputError, 'Invalid output')


class InvalidDriverError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidDriverError, self).__init__(InvalidDriverError, 'Invalid driver')


class NotAvailableError(LibTiePieException):
    """"""

    def __init__(self):
        super(NotAvailableError, self).__init__(NotAvailableError, 'Not available')


class InvalidFirmwareError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidFirmwareError, self).__init__(InvalidFirmwareError, 'Invalid firmware')


class InvalidIndexError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidIndexError, self).__init__(InvalidIndexError, 'Invalid index')


class InvalidEepromError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidEepromError, self).__init__(InvalidEepromError, 'Invalid eeprom')


class InitializationFailedError(LibTiePieException):
    """"""

    def __init__(self):
        super(InitializationFailedError, self).__init__(InitializationFailedError, 'Initialization failed')


class LibraryNotInitializedError(LibTiePieException):
    """"""

    def __init__(self):
        super(LibraryNotInitializedError, self).__init__(LibraryNotInitializedError, 'Library not initialized')


class NoTriggerEnabledError(LibTiePieException):
    """"""

    def __init__(self):
        super(NoTriggerEnabledError, self).__init__(NoTriggerEnabledError, 'No trigger enabled')


class SynchronizationFailedError(LibTiePieException):
    """"""

    def __init__(self):
        super(SynchronizationFailedError, self).__init__(SynchronizationFailedError, 'Synchronization failed')


class InvalidHS56CombinedDeviceError(LibTiePieException):
    """"""

    def __init__(self):
        super(InvalidHS56CombinedDeviceError, self).__init__(InvalidHS56CombinedDeviceError, 'Invalid hs56 combined device')


class MeasurementRunningError(LibTiePieException):
    """"""

    def __init__(self):
        super(MeasurementRunningError, self).__init__(MeasurementRunningError, 'Measurement running')


class InitializationError10001(LibTiePieException):
    """"""

    def __init__(self):
        super(InitializationError10001, self).__init__(InitializationError10001, 'Initialization error 10001')


class InitializationError10002(LibTiePieException):
    """"""

    def __init__(self):
        super(InitializationError10002, self).__init__(InitializationError10002, 'Initialization error 10002')


class InitializationError10003(LibTiePieException):
    """"""

    def __init__(self):
        super(InitializationError10003, self).__init__(InitializationError10003, 'Initialization error 10003')


class InitializationError10004(LibTiePieException):
    """"""

    def __init__(self):
        super(InitializationError10004, self).__init__(InitializationError10004, 'Initialization error 10004')


class InitializationError10005(LibTiePieException):
    """"""

    def __init__(self):
        super(InitializationError10005, self).__init__(InitializationError10005, 'Initialization error 10005')


class InitializationError10006(LibTiePieException):
    """"""

    def __init__(self):
        super(InitializationError10006, self).__init__(InitializationError10006, 'Initialization error 10006')



from ctypes import c_uint32
from .api import api
from .const import *
from .library import library
from .devicelistitem import DeviceListItem
from .exceptions import InvalidDeviceIndexError


class DeviceList(object):
    """"""

    def __getitem__(self, index):
        try:
            return self.get_item_by_index(index)
        except (InvalidDeviceIndexError):
            raise IndexError()

    def __len__(self):
        return self.count

    def get_item_by_index(self, index):
        serial_number = api.LstDevGetSerialNumber(IDKIND_INDEX, index)
        library.check_last_status_raise_on_error()
        return DeviceListItem(serial_number)

    def get_item_by_product_id(self, pid):
        serial_number = api.LstDevGetSerialNumber(IDKIND_PRODUCTID, pid)
        library.check_last_status_raise_on_error()
        return DeviceListItem(serial_number)

    def get_item_by_serial_number(self, serial_number):
        serial_number = api.LstDevGetSerialNumber(IDKIND_SERIALNUMBER, serial_number)
        library.check_last_status_raise_on_error()
        return DeviceListItem(serial_number)

    def update(self):
        api.LstUpdate()
        library.check_last_status_raise_on_error()

    def _get_count(self):
        """ Number of devices in the device list. """
        value = api.LstGetCount()
        library.check_last_status_raise_on_error()
        return value

    def create_combined_device(self, devices):
        """ Create a combined instrument.

        :param device: :class:`list` of Device instances.
        :returns: Device list item of combined device.
        """
        handles = (c_uint32 * len(devices))()
        i = 0
        for device in devices:
            handles[i] = device._handle
            i += 1
        serial_number = api.LstCreateCombinedDevice(handles, len(handles))
        library.check_last_status_raise_on_error()
        return DeviceListItem(serial_number)

    def create_and_open_combined_device(self, devices):
        """ Create and open a combined instrument.

        :param device: :class:`list` of Device instances.
        :returns: Instance of combined device.
        """
        item = self.create_combined_device(devices)
        return item.open_device(item.types)

    def remove_device(self, serial_number, force=False):
        """ Remove an instrument from the device list so it can be used by other applications.

        :param serial_number: Serial number of the device to remove.
        :param force: Force the removal, even when the device is currenty opened.
        """
        if force:
            api.LstRemoveDeviceForce(serial_number)
        else:
            api.LstRemoveDevice(serial_number)
        library.check_last_status_raise_on_error()

    def set_callback_device_added(self, callback, data):
        """ Set a callback function which is called when a device is added to the device list.

        :param callback: A pointer to the callback function. Use ``None`` to disable.
        :param data: Optional user data.
        """
        api.LstSetCallbackDeviceAdded(callback, data)
        library.check_last_status_raise_on_error()

    def set_callback_device_removed(self, callback, data):
        """ Set a callback function which is called when a device is removed from the device list.

        :param callback: A pointer to the callback function. Use ``None`` to disable.
        :param data: Optional user data.
        """
        api.LstSetCallbackDeviceRemoved(callback, data)
        library.check_last_status_raise_on_error()

    def set_callback_device_can_open_changed(self, callback, data):
        """ Set a callback function which is called when the device can open property changes.

        :param callback: A pointer to the callback function. Use ``None`` to disable.
        :param data: Optional user data.
        .. versionadded:: 0.6
        """
        api.LstSetCallbackDeviceCanOpenChanged(callback, data)
        library.check_last_status_raise_on_error()

    if platform.system() == 'Linux':
        def set_event_device_added(self, event):
            """ Set an event file descriptor which is set when a device is added to the device list.

            :param event: An event file descriptor. Use ``<0`` to disable.
            """
            api.LstSetEventDeviceAdded(event)
            library.check_last_status_raise_on_error()

        def set_event_device_removed(self, event):
            """ Set an event file descriptor which is set when a device is removed from the device list.

            :param event: an event file descriptor. Use ``<0`` to disable.
            """
            api.LstSetEventDeviceRemoved(event)
            library.check_last_status_raise_on_error()

        def set_event_device_can_open_changed(self, event):
            """ Set an event file descriptor which is set when the device can open property changes.

            :param event: an event file descriptor. Use ``<0`` to disable.
            .. versionadded:: 0.6
            """
            api.LstSetEventDeviceCanOpenChanged(event)
            library.check_last_status_raise_on_error()

    if platform.system() == 'Windows':
        def set_event_device_added(self, event):
            """ Set an event object handle which is set when a device is added to the device list.

            :param event: A handle to the event object. Use ``None`` to disable.
            """
            api.LstSetEventDeviceAdded(event)
            library.check_last_status_raise_on_error()

        def set_event_device_removed(self, event):
            """ Set an event object handle which is set when a device is removed from the device list.

            :param event: A handle to the event object. Use ``None`` to disable.
            """
            api.LstSetEventDeviceRemoved(event)
            library.check_last_status_raise_on_error()

        def set_event_device_can_open_changed(self, event):
            """ Set an event object handle which is set when the device can open property changes.

            :param event: A handle to the event object. Use ``None`` to disable.
            .. versionadded:: 0.6
            """
            api.LstSetEventDeviceCanOpenChanged(event)
            library.check_last_status_raise_on_error()

        def set_message_device_added(self, wnd):
            """ Set a window handle to which a #WM_LIBTIEPIE_LST_DEVICEADDED message is sent when a device is added to the device list.

            :param wnd: A handle to the window whose window procedure is to receive the message. Use ``None`` to disable.
            """
            api.LstSetMessageDeviceAdded(wnd)
            library.check_last_status_raise_on_error()

        def set_message_device_removed(self, wnd):
            """ Set a window handle to which a #WM_LIBTIEPIE_LST_DEVICEREMOVED message is sent when a device is removed from the device list.

            :param wnd: A handle to the window whose window procedure is to receive the message. Use ``None`` to disable.
            """
            api.LstSetMessageDeviceRemoved(wnd)
            library.check_last_status_raise_on_error()

        def set_message_device_can_open_changed(self, wnd):
            """ Set a window handle to which a #WM_LIBTIEPIE_LST_DEVICEREMOVED message is sent when the device can open property changes.

            :param wnd: A handle to the window whose window procedure is to receive the message. Use ``None`` to disable.
            .. versionadded:: 0.6
            """
            api.LstSetMessageDeviceCanOpenChanged(wnd)
            library.check_last_status_raise_on_error()

    count = property(_get_count)


device_list = DeviceList()



from collections import namedtuple
from ctypes import *


Callback = CFUNCTYPE(None, c_void_p)
CallbackDeviceList = CFUNCTYPE(None, c_void_p, c_uint32, c_uint32)
CallbackHandle = CFUNCTYPE(None, c_void_p, c_uint32)
CallbackEvent = CFUNCTYPE(None, c_void_p, c_uint32, c_uint32)


class Tristate(object):  # See: http://stackoverflow.com/a/9504358
    def __init__(self, value=None):
        if any(value is v for v in (True, False, None)):
            self.value = value
        else:
            raise ValueError('Tristate value must be True, False, or None')

    def __eq__(self, other):
        return self.value is other

    def __ne__(self, other):
        return self.value is not other

    def __nonzero__(self):
        raise TypeError('Tristate value may not be used as implicit Boolean')

    def __str__(self):
        return str(self.value)

    def __repr__(self):
        return "Tristate(%s)" % self.value


class Version(namedtuple('Version', ['major', 'minor', 'release', 'build'])):
    def __str__(self):
        return '{0:d}.{1:d}.{2:d}.{3:d}'.format(self.major, self.minor, self.release, self.build)



from array import array
from ctypes import *
from .api import api
from .const import *
from .library import library
from .device import Device


class I2CHost(Device):
    """"""

    def __init__(self, handle):
        super(I2CHost, self).__init__(handle)

    def is_internal_address(self, address):
        """ Check whether an address is used internally.

        :param address: An I2C device address.
        :returns: ``True`` if the address is used internally, ``False`` otherwise.
        """
        result = api.I2CIsInternalAddress(self._handle, address)
        library.check_last_status_raise_on_error()
        return result != BOOL8_FALSE

    def _get_internal_addresses(self):
        """ :class:`array.array` of addresses which are used internally. """
        count = api.I2CGetInternalAddresses(self._handle, None, 0)
        library.check_last_status_raise_on_error()
        values = (c_uint16 * count)()
        api.I2CGetInternalAddresses(self._handle, values, count)
        library.check_last_status_raise_on_error()
        return array('H', values)

    def read(self, address, count, stop=True):
        """ Read data from a specified address on the I2C bus.

        :param address: The I2C address to read from.
        :param count: Number of bytes to read.
        :param stop: Indicates whether an I2C stop is generated after the transaction, when ``True``, an I2C stop is generated, when ``False`` not.
        :returns: :class:`array.array` of read bytes.
        """
        buffer = (c_ubyte * count)()
        api.I2CRead(self._handle, address, pointer(buffer), count, BOOL8_TRUE if stop else BOOL8_FALSE)
        library.check_last_status_raise_on_error()
        return array('B', buffer)

    def read_byte(self, address):
        """ Read one byte from a specified address on the I2C bus.

        :param address: The I2C address to read from.
        :returns: The read value.
        """
        value = c_ubyte()
        api.I2CReadByte(self._handle, address, pointer(value))
        library.check_last_status_raise_on_error()
        return value.value

    def read_word(self, address):
        """ Read one word from a specified address on the I2C bus.

        :param address: The I2C address to read from.
        :returns: The read value.
        """
        value = c_ushort()
        api.I2CReadWord(self._handle, address, pointer(value))
        library.check_last_status_raise_on_error()
        return value.value

    def write(self, address, buffer, stop=True):
        """ Write data to a specified address on the I2C bus.

        :param address: The I2C address to write to.
        :param buffer: :class:`array.array` of bytes to write.
        :param stop: Indicates whether an I2C stop is generated after the transaction, when ``True``, an I2C stop is generated, when ``False`` not.
        """
        if isinstance(buffer, array) and buffer.typecode == 'B':
            buffer_ptr = c_void_p(buffer.buffer_info()[0])
            buffer_ptr.__ref = buffer
            api.I2CWrite(self._handle, address, buffer_ptr, len(buffer), BOOL8_TRUE if stop else BOOL8_FALSE)
            library.check_last_status_raise_on_error()
        else:
            raise Exception('Invalid data, must be array.array with typecode = `B`')

    def write_byte(self, address, value):
        """ Write one byte to a specified address on the I2C bus.

        :param address: The I2C address to write to.
        :param value: The byte value to write.
        """
        api.I2CWriteByte(self._handle, address, value)
        library.check_last_status_raise_on_error()

    def write_byte_byte(self, address, value1, value2):
        """ Write two bytes to a specified address on the I2C bus.

        :param address: The I2C address to write to.
        :param value1: The first byte value to write.
        :param value2: The second byte value to write.
        """
        api.I2CWriteByteByte(self._handle, address, value1, value2)
        library.check_last_status_raise_on_error()

    def write_byte_word(self, address, value1, value2):
        """ Write one byte and one word to a specified address on the I2C bus.

        :param address: The I2C address to write to.
        :param value1: The byte value to write.
        :param value2: The word value to write.
        """
        api.I2CWriteByteWord(self._handle, address, value1, value2)
        library.check_last_status_raise_on_error()

    def write_word(self, address, value):
        """ Write one word to a specified address on the I2C bus.

        :param address: The I2C address to write to.
        :param value: The word value to write.
        """
        api.I2CWriteWord(self._handle, address, value)
        library.check_last_status_raise_on_error()

    def write_read(self, address, write_buffer, read_count):
        """ Write and read data to/from to a specified address on the I2C bus.

        :param address: The I2C address to write to.
        :param write_buffer: :class:`array.array` of bytes to write.
        :param read_count: Number of bytes to read.
        :returns: :class:`array.array` of read bytes.
        .. versionadded:: 0.6
        """
        if isinstance(write_buffer, array) and write_buffer.typecode == 'B':
            write_buffer_ptr = c_void_p(write_buffer.buffer_info()[0])
            write_buffer_ptr.__ref = write_buffer
            read_buffer = (c_ubyte * read_count)()
            api.I2CWriteRead(self._handle, address, write_buffer_ptr, len(write_buffer), pointer(read_buffer), read_count)
            library.check_last_status_raise_on_error()
            return array('B', read_buffer)
        else:
            raise Exception('Invalid data, must be array.array with typecode = `B`')

    def _get_speed_max(self):
        """ Maximum clock speed on the I2C bus. """
        value = api.I2CGetSpeedMax(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _get_speed(self):
        """ Current clock speed on the I2C bus. """
        value = api.I2CGetSpeed(self._handle)
        library.check_last_status_raise_on_error()
        return value

    def _set_speed(self, value):
        api.I2CSetSpeed(self._handle, value)
        library.check_last_status_raise_on_error()

    def verify_speed(self, speed):
        """ Verify if a clock speed on the I2C bus can be set, without actually setting the hardware.

        :param speed: The requested I2C clock speed in Hz.
        :returns: The I2C clock speed that would have been set, in Hz.
        """
        result = api.I2CVerifySpeed(self._handle, speed)
        library.check_last_status_raise_on_error()
        return result

    internal_addresses = property(_get_internal_addresses)
    speed_max = property(_get_speed_max)
    speed = property(_get_speed, _set_speed)



from .objectlist import ObjectList
from .api import api
from .library import library
from .triggerinput import TriggerInput


class TriggerInputs(ObjectList):
    """"""

    def __init__(self, handle):
        super(TriggerInputs, self).__init__()
        self._handle = handle
        self._items = [TriggerInput(handle, i)
                       for i in range(api.DevTrGetInputCount(handle))]

    def get_by_id(self, id):
        index = api.DevTrGetInputIndexById(self._handle, id)
        library.check_last_status_raise_on_error()
        if index < len(self._items):
            return self._items[index]
        return None



import sys
from ctypes import *
import platform
from .types import *
from ._platform import DLL_PATH, is_windows


def load_libtiepie():
    """Load libtiepie library and import all functions."""

    if is_windows:
        from ctypes.wintypes import HANDLE, HWND, LPARAM, WPARAM

    api = CDLL(DLL_PATH)

    api.LibInit.restype = None
    api.LibInit.argtypes = []
    api.LibIsInitialized.restype = c_uint8
    api.LibIsInitialized.argtypes = []
    api.LibExit.restype = None
    api.LibExit.argtypes = []
    api.LibGetVersion.restype = c_uint64
    api.LibGetVersion.argtypes = []
    api.LibGetVersionExtra.restype = c_char_p
    api.LibGetVersionExtra.argtypes = []
    api.LibGetConfig.restype = c_uint32
    api.LibGetConfig.argtypes = [c_void_p, c_uint32]
    api.LibGetLastStatus.restype = c_int32
    api.LibGetLastStatus.argtypes = []
    api.LibGetLastStatusStr.restype = c_char_p
    api.LibGetLastStatusStr.argtypes = []
    api.LstUpdate.restype = None
    api.LstUpdate.argtypes = []
    api.LstGetCount.restype = c_uint32
    api.LstGetCount.argtypes = []
    api.LstOpenDevice.restype = c_uint32
    api.LstOpenDevice.argtypes = [c_uint32, c_uint32, c_uint32]
    api.LstOpenOscilloscope.restype = c_uint32
    api.LstOpenOscilloscope.argtypes = [c_uint32, c_uint32]
    api.LstOpenGenerator.restype = c_uint32
    api.LstOpenGenerator.argtypes = [c_uint32, c_uint32]
    api.LstOpenI2CHost.restype = c_uint32
    api.LstOpenI2CHost.argtypes = [c_uint32, c_uint32]
    api.LstCreateCombinedDevice.restype = c_uint32
    api.LstCreateCombinedDevice.argtypes = [c_void_p, c_uint32]
    api.LstCreateAndOpenCombinedDevice.restype = c_uint32
    api.LstCreateAndOpenCombinedDevice.argtypes = [c_void_p, c_uint32]
    api.LstRemoveDevice.restype = None
    api.LstRemoveDevice.argtypes = [c_uint32]
    api.LstRemoveDeviceForce.restype = None
    api.LstRemoveDeviceForce.argtypes = [c_uint32]
    api.LstDevCanOpen.restype = c_uint8
    api.LstDevCanOpen.argtypes = [c_uint32, c_uint32, c_uint32]
    api.LstDevGetProductId.restype = c_uint32
    api.LstDevGetProductId.argtypes = [c_uint32, c_uint32]
    api.LstDevGetVendorId.restype = c_uint32
    api.LstDevGetVendorId.argtypes = [c_uint32, c_uint32]
    api.LstDevGetName.restype = c_uint32
    api.LstDevGetName.argtypes = [c_uint32, c_uint32, c_char_p, c_uint32]
    api.LstDevGetNameShort.restype = c_uint32
    api.LstDevGetNameShort.argtypes = [c_uint32, c_uint32, c_char_p, c_uint32]
    api.LstDevGetNameShortest.restype = c_uint32
    api.LstDevGetNameShortest.argtypes = [c_uint32, c_uint32, c_char_p, c_uint32]
    api.LstDevGetDriverVersion.restype = c_uint64
    api.LstDevGetDriverVersion.argtypes = [c_uint32, c_uint32]
    api.LstDevGetRecommendedDriverVersion.restype = c_uint64
    api.LstDevGetRecommendedDriverVersion.argtypes = [c_uint32, c_uint32]
    api.LstDevGetFirmwareVersion.restype = c_uint64
    api.LstDevGetFirmwareVersion.argtypes = [c_uint32, c_uint32]
    api.LstDevGetRecommendedFirmwareVersion.restype = c_uint64
    api.LstDevGetRecommendedFirmwareVersion.argtypes = [c_uint32, c_uint32]
    api.LstDevGetCalibrationDate.restype = c_uint32
    api.LstDevGetCalibrationDate.argtypes = [c_uint32, c_uint32]
    api.LstDevGetSerialNumber.restype = c_uint32
    api.LstDevGetSerialNumber.argtypes = [c_uint32, c_uint32]
    api.LstDevGetIPv4Address.restype = c_uint32
    api.LstDevGetIPv4Address.argtypes = [c_uint32, c_uint32]
    api.LstDevGetIPPort.restype = c_uint16
    api.LstDevGetIPPort.argtypes = [c_uint32, c_uint32]
    api.LstDevHasServer.restype = c_uint8
    api.LstDevHasServer.argtypes = [c_uint32, c_uint32]
    api.LstDevGetServer.restype = c_uint32
    api.LstDevGetServer.argtypes = [c_uint32, c_uint32]
    api.LstDevGetTypes.restype = c_uint32
    api.LstDevGetTypes.argtypes = [c_uint32, c_uint32]
    api.LstDevGetContainedSerialNumbers.restype = c_uint32
    api.LstDevGetContainedSerialNumbers.argtypes = [c_uint32, c_uint32, c_void_p, c_uint32]
    api.LstCbDevGetProductId.restype = c_uint32
    api.LstCbDevGetProductId.argtypes = [c_uint32, c_uint32, c_uint32]
    api.LstCbDevGetVendorId.restype = c_uint32
    api.LstCbDevGetVendorId.argtypes = [c_uint32, c_uint32, c_uint32]
    api.LstCbDevGetName.restype = c_uint32
    api.LstCbDevGetName.argtypes = [c_uint32, c_uint32, c_uint32, c_char_p, c_uint32]
    api.LstCbDevGetNameShort.restype = c_uint32
    api.LstCbDevGetNameShort.argtypes = [c_uint32, c_uint32, c_uint32, c_char_p, c_uint32]
    api.LstCbDevGetNameShortest.restype = c_uint32
    api.LstCbDevGetNameShortest.argtypes = [c_uint32, c_uint32, c_uint32, c_char_p, c_uint32]
    api.LstCbDevGetDriverVersion.restype = c_uint64
    api.LstCbDevGetDriverVersion.argtypes = [c_uint32, c_uint32, c_uint32]
    api.LstCbDevGetFirmwareVersion.restype = c_uint64
    api.LstCbDevGetFirmwareVersion.argtypes = [c_uint32, c_uint32, c_uint32]
    api.LstCbDevGetCalibrationDate.restype = c_uint32
    api.LstCbDevGetCalibrationDate.argtypes = [c_uint32, c_uint32, c_uint32]
    api.LstCbScpGetChannelCount.restype = c_uint16
    api.LstCbScpGetChannelCount.argtypes = [c_uint32, c_uint32, c_uint32]
    api.LstSetCallbackDeviceAdded.restype = None
    api.LstSetCallbackDeviceAdded.argtypes = [CallbackDeviceList, c_void_p]
    api.LstSetCallbackDeviceRemoved.restype = None
    api.LstSetCallbackDeviceRemoved.argtypes = [CallbackDeviceList, c_void_p]
    api.LstSetCallbackDeviceCanOpenChanged.restype = None
    api.LstSetCallbackDeviceCanOpenChanged.argtypes = [CallbackDeviceList, c_void_p]
    if platform.system() == 'Linux':
        api.LstSetEventDeviceAdded.restype = None
        api.LstSetEventDeviceAdded.argtypes = [c_int]
        api.LstSetEventDeviceRemoved.restype = None
        api.LstSetEventDeviceRemoved.argtypes = [c_int]
        api.LstSetEventDeviceCanOpenChanged.restype = None
        api.LstSetEventDeviceCanOpenChanged.argtypes = [c_int]
    if platform.system() == 'Windows':
        api.LstSetEventDeviceAdded.restype = None
        api.LstSetEventDeviceAdded.argtypes = [HANDLE]
        api.LstSetEventDeviceRemoved.restype = None
        api.LstSetEventDeviceRemoved.argtypes = [HANDLE]
        api.LstSetEventDeviceCanOpenChanged.restype = None
        api.LstSetEventDeviceCanOpenChanged.argtypes = [HANDLE]
        api.LstSetMessageDeviceAdded.restype = None
        api.LstSetMessageDeviceAdded.argtypes = [HWND]
        api.LstSetMessageDeviceRemoved.restype = None
        api.LstSetMessageDeviceRemoved.argtypes = [HWND]
        api.LstSetMessageDeviceCanOpenChanged.restype = None
        api.LstSetMessageDeviceCanOpenChanged.argtypes = [HWND]
    api.NetGetAutoDetectEnabled.restype = c_uint8
    api.NetGetAutoDetectEnabled.argtypes = []
    api.NetSetAutoDetectEnabled.restype = c_uint8
    api.NetSetAutoDetectEnabled.argtypes = [c_uint8]
    api.NetSrvAdd.restype = c_uint8
    api.NetSrvAdd.argtypes = [c_char_p, c_uint32, c_void_p]
    api.NetSrvRemove.restype = c_uint8
    api.NetSrvRemove.argtypes = [c_char_p, c_uint32, c_uint8]
    api.NetSrvGetCount.restype = c_uint32
    api.NetSrvGetCount.argtypes = []
    api.NetSrvGetByIndex.restype = c_uint32
    api.NetSrvGetByIndex.argtypes = [c_uint32]
    api.NetSrvGetByURL.restype = c_uint32
    api.NetSrvGetByURL.argtypes = [c_char_p, c_uint32]
    api.NetSrvSetCallbackAdded.restype = None
    api.NetSrvSetCallbackAdded.argtypes = [CallbackHandle, c_void_p]
    if platform.system() == 'Linux':
        api.NetSrvSetEventAdded.restype = None
        api.NetSrvSetEventAdded.argtypes = [c_int]
    if platform.system() == 'Windows':
        api.NetSrvSetEventAdded.restype = None
        api.NetSrvSetEventAdded.argtypes = [HANDLE]
        api.NetSrvSetMessageAdded.restype = None
        api.NetSrvSetMessageAdded.argtypes = [HWND]
    api.ObjClose.restype = None
    api.ObjClose.argtypes = [c_uint32]
    api.ObjIsRemoved.restype = c_uint8
    api.ObjIsRemoved.argtypes = [c_uint32]
    api.ObjGetInterfaces.restype = c_uint64
    api.ObjGetInterfaces.argtypes = [c_uint32]
    api.ObjSetEventCallback.restype = None
    api.ObjSetEventCallback.argtypes = [c_uint32, CallbackEvent, c_void_p]
    api.ObjGetEvent.restype = c_uint8
    api.ObjGetEvent.argtypes = [c_uint32, c_void_p, c_void_p]
    if platform.system() == 'Linux':
        api.ObjSetEventEvent.restype = None
        api.ObjSetEventEvent.argtypes = [c_uint32, c_int]
    if platform.system() == 'Windows':
        api.ObjSetEventEvent.restype = None
        api.ObjSetEventEvent.argtypes = [c_uint32, HANDLE]
        api.ObjSetEventWindowHandle.restype = None
        api.ObjSetEventWindowHandle.argtypes = [c_uint32, HWND]
    api.DevClose.restype = None
    api.DevClose.argtypes = [c_uint32]
    api.DevIsRemoved.restype = c_uint8
    api.DevIsRemoved.argtypes = [c_uint32]
    api.DevGetDriverVersion.restype = c_uint64
    api.DevGetDriverVersion.argtypes = [c_uint32]
    api.DevGetFirmwareVersion.restype = c_uint64
    api.DevGetFirmwareVersion.argtypes = [c_uint32]
    api.DevGetCalibrationDate.restype = c_uint32
    api.DevGetCalibrationDate.argtypes = [c_uint32]
    api.DevGetCalibrationToken.restype = c_uint32
    api.DevGetCalibrationToken.argtypes = [c_uint32, c_char_p, c_uint32]
    api.DevGetSerialNumber.restype = c_uint32
    api.DevGetSerialNumber.argtypes = [c_uint32]
    api.DevGetIPv4Address.restype = c_uint32
    api.DevGetIPv4Address.argtypes = [c_uint32]
    api.DevGetIPPort.restype = c_uint16
    api.DevGetIPPort.argtypes = [c_uint32]
    api.DevGetProductId.restype = c_uint32
    api.DevGetProductId.argtypes = [c_uint32]
    api.DevGetVendorId.restype = c_uint32
    api.DevGetVendorId.argtypes = [c_uint32]
    api.DevGetType.restype = c_uint32
    api.DevGetType.argtypes = [c_uint32]
    api.DevGetName.restype = c_uint32
    api.DevGetName.argtypes = [c_uint32, c_char_p, c_uint32]
    api.DevGetNameShort.restype = c_uint32
    api.DevGetNameShort.argtypes = [c_uint32, c_char_p, c_uint32]
    api.DevGetNameShortest.restype = c_uint32
    api.DevGetNameShortest.argtypes = [c_uint32, c_char_p, c_uint32]
    api.DevHasBattery.restype = c_uint8
    api.DevHasBattery.argtypes = [c_uint32]
    api.DevGetBatteryCharge.restype = c_int8
    api.DevGetBatteryCharge.argtypes = [c_uint32]
    api.DevGetBatteryTimeToEmpty.restype = c_int32
    api.DevGetBatteryTimeToEmpty.argtypes = [c_uint32]
    api.DevGetBatteryTimeToFull.restype = c_int32
    api.DevGetBatteryTimeToFull.argtypes = [c_uint32]
    api.DevIsBatteryChargerConnected.restype = c_uint8
    api.DevIsBatteryChargerConnected.argtypes = [c_uint32]
    api.DevIsBatteryCharging.restype = c_uint8
    api.DevIsBatteryCharging.argtypes = [c_uint32]
    api.DevIsBatteryBroken.restype = c_uint8
    api.DevIsBatteryBroken.argtypes = [c_uint32]
    api.DevSetCallbackRemoved.restype = None
    api.DevSetCallbackRemoved.argtypes = [c_uint32, Callback, c_void_p]
    if platform.system() == 'Linux':
        api.DevSetEventRemoved.restype = None
        api.DevSetEventRemoved.argtypes = [c_uint32, c_int]
    if platform.system() == 'Windows':
        api.DevSetEventRemoved.restype = None
        api.DevSetEventRemoved.argtypes = [c_uint32, HANDLE]
        api.DevSetMessageRemoved.restype = None
        api.DevSetMessageRemoved.argtypes = [c_uint32, HWND, WPARAM, LPARAM]
    api.DevTrGetInputCount.restype = c_uint16
    api.DevTrGetInputCount.argtypes = [c_uint32]
    api.DevTrGetInputIndexById.restype = c_uint16
    api.DevTrGetInputIndexById.argtypes = [c_uint32, c_uint32]
    api.ScpTrInIsTriggered.restype = c_uint8
    api.ScpTrInIsTriggered.argtypes = [c_uint32, c_uint16]
    api.DevTrInGetEnabled.restype = c_uint8
    api.DevTrInGetEnabled.argtypes = [c_uint32, c_uint16]
    api.DevTrInSetEnabled.restype = c_uint8
    api.DevTrInSetEnabled.argtypes = [c_uint32, c_uint16, c_uint8]
    api.DevTrInGetKinds.restype = c_uint64
    api.DevTrInGetKinds.argtypes = [c_uint32, c_uint16]
    api.ScpTrInGetKindsEx.restype = c_uint64
    api.ScpTrInGetKindsEx.argtypes = [c_uint32, c_uint16, c_uint32]
    api.DevTrInGetKind.restype = c_uint64
    api.DevTrInGetKind.argtypes = [c_uint32, c_uint16]
    api.DevTrInSetKind.restype = c_uint64
    api.DevTrInSetKind.argtypes = [c_uint32, c_uint16, c_uint64]
    api.DevTrInIsAvailable.restype = c_uint8
    api.DevTrInIsAvailable.argtypes = [c_uint32, c_uint16]
    api.ScpTrInIsAvailableEx.restype = c_uint8
    api.ScpTrInIsAvailableEx.argtypes = [c_uint32, c_uint16, c_uint32]
    api.DevTrInGetId.restype = c_uint32
    api.DevTrInGetId.argtypes = [c_uint32, c_uint16]
    api.DevTrInGetName.restype = c_uint32
    api.DevTrInGetName.argtypes = [c_uint32, c_uint16, c_char_p, c_uint32]
    api.DevTrGetOutputCount.restype = c_uint16
    api.DevTrGetOutputCount.argtypes = [c_uint32]
    api.DevTrGetOutputIndexById.restype = c_uint16
    api.DevTrGetOutputIndexById.argtypes = [c_uint32, c_uint32]
    api.DevTrOutGetEnabled.restype = c_uint8
    api.DevTrOutGetEnabled.argtypes = [c_uint32, c_uint16]
    api.DevTrOutSetEnabled.restype = c_uint8
    api.DevTrOutSetEnabled.argtypes = [c_uint32, c_uint16, c_uint8]
    api.DevTrOutGetEvents.restype = c_uint64
    api.DevTrOutGetEvents.argtypes = [c_uint32, c_uint16]
    api.DevTrOutGetEvent.restype = c_uint64
    api.DevTrOutGetEvent.argtypes = [c_uint32, c_uint16]
    api.DevTrOutSetEvent.restype = c_uint64
    api.DevTrOutSetEvent.argtypes = [c_uint32, c_uint16, c_uint64]
    api.DevTrOutGetId.restype = c_uint32
    api.DevTrOutGetId.argtypes = [c_uint32, c_uint16]
    api.DevTrOutGetName.restype = c_uint32
    api.DevTrOutGetName.argtypes = [c_uint32, c_uint16, c_char_p, c_uint32]
    api.DevTrOutTrigger.restype = c_uint8
    api.DevTrOutTrigger.argtypes = [c_uint32, c_uint16]
    api.ScpGetChannelCount.restype = c_uint16
    api.ScpGetChannelCount.argtypes = [c_uint32]
    api.ScpChIsAvailable.restype = c_uint8
    api.ScpChIsAvailable.argtypes = [c_uint32, c_uint16]
    api.ScpChIsAvailableEx.restype = c_uint8
    api.ScpChIsAvailableEx.argtypes = [c_uint32, c_uint16, c_uint32, c_double, c_uint8, c_void_p, c_uint16]
    api.ScpChGetConnectorType.restype = c_uint32
    api.ScpChGetConnectorType.argtypes = [c_uint32, c_uint16]
    api.ScpChIsDifferential.restype = c_uint8
    api.ScpChIsDifferential.argtypes = [c_uint32, c_uint16]
    api.ScpChGetImpedance.restype = c_double
    api.ScpChGetImpedance.argtypes = [c_uint32, c_uint16]
    api.ScpChGetBandwidths.restype = c_uint32
    api.ScpChGetBandwidths.argtypes = [c_uint32, c_uint16, c_void_p, c_uint32]
    api.ScpChGetBandwidth.restype = c_double
    api.ScpChGetBandwidth.argtypes = [c_uint32, c_uint16]
    api.ScpChSetBandwidth.restype = c_double
    api.ScpChSetBandwidth.argtypes = [c_uint32, c_uint16, c_double]
    api.ScpChGetCouplings.restype = c_uint64
    api.ScpChGetCouplings.argtypes = [c_uint32, c_uint16]
    api.ScpChGetCoupling.restype = c_uint64
    api.ScpChGetCoupling.argtypes = [c_uint32, c_uint16]
    api.ScpChSetCoupling.restype = c_uint64
    api.ScpChSetCoupling.argtypes = [c_uint32, c_uint16, c_uint64]
    api.ScpChGetEnabled.restype = c_uint8
    api.ScpChGetEnabled.argtypes = [c_uint32, c_uint16]
    api.ScpChSetEnabled.restype = c_uint8
    api.ScpChSetEnabled.argtypes = [c_uint32, c_uint16, c_uint8]
    api.ScpChGetProbeGain.restype = c_double
    api.ScpChGetProbeGain.argtypes = [c_uint32, c_uint16]
    api.ScpChSetProbeGain.restype = c_double
    api.ScpChSetProbeGain.argtypes = [c_uint32, c_uint16, c_double]
    api.ScpChGetProbeOffset.restype = c_double
    api.ScpChGetProbeOffset.argtypes = [c_uint32, c_uint16]
    api.ScpChSetProbeOffset.restype = c_double
    api.ScpChSetProbeOffset.argtypes = [c_uint32, c_uint16, c_double]
    api.ScpChGetAutoRanging.restype = c_uint8
    api.ScpChGetAutoRanging.argtypes = [c_uint32, c_uint16]
    api.ScpChSetAutoRanging.restype = c_uint8
    api.ScpChSetAutoRanging.argtypes = [c_uint32, c_uint16, c_uint8]
    api.ScpChGetRanges.restype = c_uint32
    api.ScpChGetRanges.argtypes = [c_uint32, c_uint16, c_void_p, c_uint32]
    api.ScpChGetRangesEx.restype = c_uint32
    api.ScpChGetRangesEx.argtypes = [c_uint32, c_uint16, c_uint64, c_void_p, c_uint32]
    api.ScpChGetRange.restype = c_double
    api.ScpChGetRange.argtypes = [c_uint32, c_uint16]
    api.ScpChSetRange.restype = c_double
    api.ScpChSetRange.argtypes = [c_uint32, c_uint16, c_double]
    api.ScpChHasSafeGround.restype = c_uint8
    api.ScpChHasSafeGround.argtypes = [c_uint32, c_uint16]
    api.ScpChGetSafeGroundEnabled.restype = c_uint8
    api.ScpChGetSafeGroundEnabled.argtypes = [c_uint32, c_uint16]
    api.ScpChSetSafeGroundEnabled.restype = c_uint8
    api.ScpChSetSafeGroundEnabled.argtypes = [c_uint32, c_uint16, c_uint8]
    api.ScpChGetSafeGroundThresholdMin.restype = c_double
    api.ScpChGetSafeGroundThresholdMin.argtypes = [c_uint32, c_uint16]
    api.ScpChGetSafeGroundThresholdMax.restype = c_double
    api.ScpChGetSafeGroundThresholdMax.argtypes = [c_uint32, c_uint16]
    api.ScpChGetSafeGroundThreshold.restype = c_double
    api.ScpChGetSafeGroundThreshold.argtypes = [c_uint32, c_uint16]
    api.ScpChSetSafeGroundThreshold.restype = c_double
    api.ScpChSetSafeGroundThreshold.argtypes = [c_uint32, c_uint16, c_double]
    api.ScpChVerifySafeGroundThreshold.restype = c_double
    api.ScpChVerifySafeGroundThreshold.argtypes = [c_uint32, c_uint16, c_double]
    api.ScpChHasTrigger.restype = c_uint8
    api.ScpChHasTrigger.argtypes = [c_uint32, c_uint16]
    api.ScpChHasTriggerEx.restype = c_uint8
    api.ScpChHasTriggerEx.argtypes = [c_uint32, c_uint16, c_uint32]
    api.ScpChTrIsAvailable.restype = c_uint8
    api.ScpChTrIsAvailable.argtypes = [c_uint32, c_uint16]
    api.ScpChTrIsAvailableEx.restype = c_uint8
    api.ScpChTrIsAvailableEx.argtypes = [c_uint32, c_uint16, c_uint32, c_double, c_uint8, c_void_p, c_void_p, c_uint16]
    api.ScpChTrIsTriggered.restype = c_uint8
    api.ScpChTrIsTriggered.argtypes = [c_uint32, c_uint16]
    api.ScpChTrGetEnabled.restype = c_uint8
    api.ScpChTrGetEnabled.argtypes = [c_uint32, c_uint16]
    api.ScpChTrSetEnabled.restype = c_uint8
    api.ScpChTrSetEnabled.argtypes = [c_uint32, c_uint16, c_uint8]
    api.ScpChTrGetKinds.restype = c_uint64
    api.ScpChTrGetKinds.argtypes = [c_uint32, c_uint16]
    api.ScpChTrGetKindsEx.restype = c_uint64
    api.ScpChTrGetKindsEx.argtypes = [c_uint32, c_uint16, c_uint32]
    api.ScpChTrGetKind.restype = c_uint64
    api.ScpChTrGetKind.argtypes = [c_uint32, c_uint16]
    api.ScpChTrSetKind.restype = c_uint64
    api.ScpChTrSetKind.argtypes = [c_uint32, c_uint16, c_uint64]
    api.ScpChTrGetLevelModes.restype = c_uint32
    api.ScpChTrGetLevelModes.argtypes = [c_uint32, c_uint16]
    api.ScpChTrGetLevelMode.restype = c_uint32
    api.ScpChTrGetLevelMode.argtypes = [c_uint32, c_uint16]
    api.ScpChTrSetLevelMode.restype = c_uint32
    api.ScpChTrSetLevelMode.argtypes = [c_uint32, c_uint16, c_uint32]
    api.ScpChTrGetLevelCount.restype = c_uint32
    api.ScpChTrGetLevelCount.argtypes = [c_uint32, c_uint16]
    api.ScpChTrGetLevel.restype = c_double
    api.ScpChTrGetLevel.argtypes = [c_uint32, c_uint16, c_uint32]
    api.ScpChTrSetLevel.restype = c_double
    api.ScpChTrSetLevel.argtypes = [c_uint32, c_uint16, c_uint32, c_double]
    api.ScpChTrGetHysteresisCount.restype = c_uint32
    api.ScpChTrGetHysteresisCount.argtypes = [c_uint32, c_uint16]
    api.ScpChTrGetHysteresis.restype = c_double
    api.ScpChTrGetHysteresis.argtypes = [c_uint32, c_uint16, c_uint32]
    api.ScpChTrSetHysteresis.restype = c_double
    api.ScpChTrSetHysteresis.argtypes = [c_uint32, c_uint16, c_uint32, c_double]
    api.ScpChTrGetConditions.restype = c_uint32
    api.ScpChTrGetConditions.argtypes = [c_uint32, c_uint16]
    api.ScpChTrGetConditionsEx.restype = c_uint32
    api.ScpChTrGetConditionsEx.argtypes = [c_uint32, c_uint16, c_uint32, c_uint64]
    api.ScpChTrGetCondition.restype = c_uint32
    api.ScpChTrGetCondition.argtypes = [c_uint32, c_uint16]
    api.ScpChTrSetCondition.restype = c_uint32
    api.ScpChTrSetCondition.argtypes = [c_uint32, c_uint16, c_uint32]
    api.ScpChTrGetTimeCount.restype = c_uint32
    api.ScpChTrGetTimeCount.argtypes = [c_uint32, c_uint16]
    api.ScpChTrGetTime.restype = c_double
    api.ScpChTrGetTime.argtypes = [c_uint32, c_uint16, c_uint32]
    api.ScpChTrSetTime.restype = c_double
    api.ScpChTrSetTime.argtypes = [c_uint32, c_uint16, c_uint32, c_double]
    api.ScpChTrVerifyTime.restype = c_double
    api.ScpChTrVerifyTime.argtypes = [c_uint32, c_uint16, c_uint32, c_double]
    api.ScpChTrVerifyTimeEx2.restype = c_double
    api.ScpChTrVerifyTimeEx2.argtypes = [c_uint32, c_uint16, c_uint32, c_double, c_uint32, c_double, c_uint64, c_uint32]
    api.ScpGetData.restype = c_uint64
    api.ScpGetData.argtypes = [c_uint32, c_void_p, c_uint16, c_uint64, c_uint64]
    api.ScpGetData1Ch.restype = c_uint64
    api.ScpGetData1Ch.argtypes = [c_uint32, c_void_p, c_uint64, c_uint64]
    api.ScpGetData2Ch.restype = c_uint64
    api.ScpGetData2Ch.argtypes = [c_uint32, c_void_p, c_void_p, c_uint64, c_uint64]
    api.ScpGetData3Ch.restype = c_uint64
    api.ScpGetData3Ch.argtypes = [c_uint32, c_void_p, c_void_p, c_void_p, c_uint64, c_uint64]
    api.ScpGetData4Ch.restype = c_uint64
    api.ScpGetData4Ch.argtypes = [c_uint32, c_void_p, c_void_p, c_void_p, c_void_p, c_uint64, c_uint64]
    api.ScpGetValidPreSampleCount.restype = c_uint64
    api.ScpGetValidPreSampleCount.argtypes = [c_uint32]
    api.ScpChGetDataValueRange.restype = None
    api.ScpChGetDataValueRange.argtypes = [c_uint32, c_uint16, c_void_p, c_void_p]
    api.ScpChGetDataValueMin.restype = c_double
    api.ScpChGetDataValueMin.argtypes = [c_uint32, c_uint16]
    api.ScpChGetDataValueMax.restype = c_double
    api.ScpChGetDataValueMax.argtypes = [c_uint32, c_uint16]
    api.ScpGetDataRaw.restype = c_uint64
    api.ScpGetDataRaw.argtypes = [c_uint32, c_void_p, c_uint16, c_uint64, c_uint64]
    api.ScpGetDataRaw1Ch.restype = c_uint64
    api.ScpGetDataRaw1Ch.argtypes = [c_uint32, c_void_p, c_uint64, c_uint64]
    api.ScpGetDataRaw2Ch.restype = c_uint64
    api.ScpGetDataRaw2Ch.argtypes = [c_uint32, c_void_p, c_void_p, c_uint64, c_uint64]
    api.ScpGetDataRaw3Ch.restype = c_uint64
    api.ScpGetDataRaw3Ch.argtypes = [c_uint32, c_void_p, c_void_p, c_void_p, c_uint64, c_uint64]
    api.ScpGetDataRaw4Ch.restype = c_uint64
    api.ScpGetDataRaw4Ch.argtypes = [c_uint32, c_void_p, c_void_p, c_void_p, c_void_p, c_uint64, c_uint64]
    api.ScpChGetDataRawType.restype = c_uint32
    api.ScpChGetDataRawType.argtypes = [c_uint32, c_uint16]
    api.ScpChGetDataRawValueRange.restype = None
    api.ScpChGetDataRawValueRange.argtypes = [c_uint32, c_uint16, c_void_p, c_void_p, c_void_p]
    api.ScpChGetDataRawValueMin.restype = c_int64
    api.ScpChGetDataRawValueMin.argtypes = [c_uint32, c_uint16]
    api.ScpChGetDataRawValueZero.restype = c_int64
    api.ScpChGetDataRawValueZero.argtypes = [c_uint32, c_uint16]
    api.ScpChGetDataRawValueMax.restype = c_int64
    api.ScpChGetDataRawValueMax.argtypes = [c_uint32, c_uint16]
    api.ScpChIsRangeMaxReachable.restype = c_uint8
    api.ScpChIsRangeMaxReachable.argtypes = [c_uint32, c_uint16]
    api.ScpIsGetDataAsyncCompleted.restype = c_uint8
    api.ScpIsGetDataAsyncCompleted.argtypes = [c_uint32]
    api.ScpStartGetDataAsync.restype = c_uint8
    api.ScpStartGetDataAsync.argtypes = [c_uint32, c_void_p, c_uint16, c_uint64, c_uint64]
    api.ScpStartGetDataAsyncRaw.restype = c_uint8
    api.ScpStartGetDataAsyncRaw.argtypes = [c_uint32, c_void_p, c_uint16, c_uint64, c_uint64]
    api.ScpCancelGetDataAsync.restype = c_uint8
    api.ScpCancelGetDataAsync.argtypes = [c_uint32]
    api.ScpSetCallbackDataReady.restype = None
    api.ScpSetCallbackDataReady.argtypes = [c_uint32, Callback, c_void_p]
    api.ScpSetCallbackDataOverflow.restype = None
    api.ScpSetCallbackDataOverflow.argtypes = [c_uint32, Callback, c_void_p]
    api.ScpSetCallbackConnectionTestCompleted.restype = None
    api.ScpSetCallbackConnectionTestCompleted.argtypes = [c_uint32, Callback, c_void_p]
    api.ScpSetCallbackTriggered.restype = None
    api.ScpSetCallbackTriggered.argtypes = [c_uint32, Callback, c_void_p]
    if platform.system() == 'Linux':
        api.ScpSetEventDataReady.restype = None
        api.ScpSetEventDataReady.argtypes = [c_uint32, c_int]
        api.ScpSetEventDataOverflow.restype = None
        api.ScpSetEventDataOverflow.argtypes = [c_uint32, c_int]
        api.ScpSetEventConnectionTestCompleted.restype = None
        api.ScpSetEventConnectionTestCompleted.argtypes = [c_uint32, c_int]
        api.ScpSetEventTriggered.restype = None
        api.ScpSetEventTriggered.argtypes = [c_uint32, c_int]
    if platform.system() == 'Windows':
        api.ScpSetEventDataReady.restype = None
        api.ScpSetEventDataReady.argtypes = [c_uint32, HANDLE]
        api.ScpSetEventDataOverflow.restype = None
        api.ScpSetEventDataOverflow.argtypes = [c_uint32, HANDLE]
        api.ScpSetEventConnectionTestCompleted.restype = None
        api.ScpSetEventConnectionTestCompleted.argtypes = [c_uint32, HANDLE]
        api.ScpSetEventTriggered.restype = None
        api.ScpSetEventTriggered.argtypes = [c_uint32, HANDLE]
        api.ScpSetMessageDataReady.restype = None
        api.ScpSetMessageDataReady.argtypes = [c_uint32, HWND, WPARAM, LPARAM]
        api.ScpSetMessageDataOverflow.restype = None
        api.ScpSetMessageDataOverflow.argtypes = [c_uint32, HWND, WPARAM, LPARAM]
        api.ScpSetMessageConnectionTestCompleted.restype = None
        api.ScpSetMessageConnectionTestCompleted.argtypes = [c_uint32, HWND, WPARAM, LPARAM]
        api.ScpSetMessageTriggered.restype = None
        api.ScpSetMessageTriggered.argtypes = [c_uint32, HWND, WPARAM, LPARAM]
    api.ScpStart.restype = c_uint8
    api.ScpStart.argtypes = [c_uint32]
    api.ScpStop.restype = c_uint8
    api.ScpStop.argtypes = [c_uint32]
    api.ScpForceTrigger.restype = c_uint8
    api.ScpForceTrigger.argtypes = [c_uint32]
    api.ScpGetMeasureModes.restype = c_uint32
    api.ScpGetMeasureModes.argtypes = [c_uint32]
    api.ScpGetMeasureMode.restype = c_uint32
    api.ScpGetMeasureMode.argtypes = [c_uint32]
    api.ScpSetMeasureMode.restype = c_uint32
    api.ScpSetMeasureMode.argtypes = [c_uint32, c_uint32]
    api.ScpIsRunning.restype = c_uint8
    api.ScpIsRunning.argtypes = [c_uint32]
    api.ScpIsTriggered.restype = c_uint8
    api.ScpIsTriggered.argtypes = [c_uint32]
    api.ScpIsTimeOutTriggered.restype = c_uint8
    api.ScpIsTimeOutTriggered.argtypes = [c_uint32]
    api.ScpIsForceTriggered.restype = c_uint8
    api.ScpIsForceTriggered.argtypes = [c_uint32]
    api.ScpIsDataReady.restype = c_uint8
    api.ScpIsDataReady.argtypes = [c_uint32]
    api.ScpIsDataOverflow.restype = c_uint8
    api.ScpIsDataOverflow.argtypes = [c_uint32]
    api.ScpGetAutoResolutionModes.restype = c_uint32
    api.ScpGetAutoResolutionModes.argtypes = [c_uint32]
    api.ScpGetAutoResolutionMode.restype = c_uint32
    api.ScpGetAutoResolutionMode.argtypes = [c_uint32]
    api.ScpSetAutoResolutionMode.restype = c_uint32
    api.ScpSetAutoResolutionMode.argtypes = [c_uint32, c_uint32]
    api.ScpGetResolutions.restype = c_uint32
    api.ScpGetResolutions.argtypes = [c_uint32, c_void_p, c_uint32]
    api.ScpGetResolution.restype = c_uint8
    api.ScpGetResolution.argtypes = [c_uint32]
    api.ScpSetResolution.restype = c_uint8
    api.ScpSetResolution.argtypes = [c_uint32, c_uint8]
    api.ScpIsResolutionEnhanced.restype = c_uint8
    api.ScpIsResolutionEnhanced.argtypes = [c_uint32]
    api.ScpIsResolutionEnhancedEx.restype = c_uint8
    api.ScpIsResolutionEnhancedEx.argtypes = [c_uint32, c_uint8]
    api.ScpGetClockSources.restype = c_uint32
    api.ScpGetClockSources.argtypes = [c_uint32]
    api.ScpGetClockSource.restype = c_uint32
    api.ScpGetClockSource.argtypes = [c_uint32]
    api.ScpSetClockSource.restype = c_uint32
    api.ScpSetClockSource.argtypes = [c_uint32, c_uint32]
    api.ScpGetClockSourceFrequencies.restype = c_uint32
    api.ScpGetClockSourceFrequencies.argtypes = [c_uint32, c_void_p, c_uint32]
    api.ScpGetClockSourceFrequenciesEx.restype = c_uint32
    api.ScpGetClockSourceFrequenciesEx.argtypes = [c_uint32, c_uint32, c_void_p, c_uint32]
    api.ScpGetClockSourceFrequency.restype = c_double
    api.ScpGetClockSourceFrequency.argtypes = [c_uint32]
    api.ScpSetClockSourceFrequency.restype = c_double
    api.ScpSetClockSourceFrequency.argtypes = [c_uint32, c_double]
    api.ScpGetClockOutputs.restype = c_uint32
    api.ScpGetClockOutputs.argtypes = [c_uint32]
    api.ScpGetClockOutput.restype = c_uint32
    api.ScpGetClockOutput.argtypes = [c_uint32]
    api.ScpSetClockOutput.restype = c_uint32
    api.ScpSetClockOutput.argtypes = [c_uint32, c_uint32]
    api.ScpGetClockOutputFrequencies.restype = c_uint32
    api.ScpGetClockOutputFrequencies.argtypes = [c_uint32, c_void_p, c_uint32]
    api.ScpGetClockOutputFrequenciesEx.restype = c_uint32
    api.ScpGetClockOutputFrequenciesEx.argtypes = [c_uint32, c_uint32, c_void_p, c_uint32]
    api.ScpGetClockOutputFrequency.restype = c_double
    api.ScpGetClockOutputFrequency.argtypes = [c_uint32]
    api.ScpSetClockOutputFrequency.restype = c_double
    api.ScpSetClockOutputFrequency.argtypes = [c_uint32, c_double]
    api.ScpGetSampleFrequencyMax.restype = c_double
    api.ScpGetSampleFrequencyMax.argtypes = [c_uint32]
    api.ScpGetSampleFrequency.restype = c_double
    api.ScpGetSampleFrequency.argtypes = [c_uint32]
    api.ScpSetSampleFrequency.restype = c_double
    api.ScpSetSampleFrequency.argtypes = [c_uint32, c_double]
    api.ScpVerifySampleFrequency.restype = c_double
    api.ScpVerifySampleFrequency.argtypes = [c_uint32, c_double]
    api.ScpVerifySampleFrequencyEx.restype = c_double
    api.ScpVerifySampleFrequencyEx.argtypes = [c_uint32, c_double, c_uint32, c_uint8, c_void_p, c_uint16]
    api.ScpVerifySampleFrequenciesEx.restype = None
    api.ScpVerifySampleFrequenciesEx.argtypes = [c_uint32, c_void_p, c_uint32, c_uint32, c_uint32, c_uint8, c_void_p, c_uint16]
    api.ScpGetRecordLengthMax.restype = c_uint64
    api.ScpGetRecordLengthMax.argtypes = [c_uint32]
    api.ScpGetRecordLengthMaxEx.restype = c_uint64
    api.ScpGetRecordLengthMaxEx.argtypes = [c_uint32, c_uint32, c_uint8]
    api.ScpGetRecordLength.restype = c_uint64
    api.ScpGetRecordLength.argtypes = [c_uint32]
    api.ScpSetRecordLength.restype = c_uint64
    api.ScpSetRecordLength.argtypes = [c_uint32, c_uint64]
    api.ScpVerifyRecordLength.restype = c_uint64
    api.ScpVerifyRecordLength.argtypes = [c_uint32, c_uint64]
    api.ScpVerifyRecordLengthEx.restype = c_uint64
    api.ScpVerifyRecordLengthEx.argtypes = [c_uint32, c_uint64, c_uint32, c_uint8, c_void_p, c_uint16]
    api.ScpGetPreSampleRatio.restype = c_double
    api.ScpGetPreSampleRatio.argtypes = [c_uint32]
    api.ScpSetPreSampleRatio.restype = c_double
    api.ScpSetPreSampleRatio.argtypes = [c_uint32, c_double]
    api.ScpGetSegmentCountMax.restype = c_uint32
    api.ScpGetSegmentCountMax.argtypes = [c_uint32]
    api.ScpGetSegmentCountMaxEx.restype = c_uint32
    api.ScpGetSegmentCountMaxEx.argtypes = [c_uint32, c_uint32]
    api.ScpGetSegmentCount.restype = c_uint32
    api.ScpGetSegmentCount.argtypes = [c_uint32]
    api.ScpSetSegmentCount.restype = c_uint32
    api.ScpSetSegmentCount.argtypes = [c_uint32, c_uint32]
    api.ScpVerifySegmentCount.restype = c_uint32
    api.ScpVerifySegmentCount.argtypes = [c_uint32, c_uint32]
    api.ScpVerifySegmentCountEx2.restype = c_uint32
    api.ScpVerifySegmentCountEx2.argtypes = [c_uint32, c_uint32, c_uint32, c_uint64, c_void_p, c_uint16]
    api.ScpHasTrigger.restype = c_uint8
    api.ScpHasTrigger.argtypes = [c_uint32]
    api.ScpHasTriggerEx.restype = c_uint8
    api.ScpHasTriggerEx.argtypes = [c_uint32, c_uint32]
    api.ScpGetTriggerTimeOut.restype = c_double
    api.ScpGetTriggerTimeOut.argtypes = [c_uint32]
    api.ScpSetTriggerTimeOut.restype = c_double
    api.ScpSetTriggerTimeOut.argtypes = [c_uint32, c_double]
    api.ScpVerifyTriggerTimeOut.restype = c_double
    api.ScpVerifyTriggerTimeOut.argtypes = [c_uint32, c_double]
    api.ScpVerifyTriggerTimeOutEx.restype = c_double
    api.ScpVerifyTriggerTimeOutEx.argtypes = [c_uint32, c_double, c_uint32, c_double]
    api.ScpHasTriggerDelay.restype = c_uint8
    api.ScpHasTriggerDelay.argtypes = [c_uint32]
    api.ScpHasTriggerDelayEx.restype = c_uint8
    api.ScpHasTriggerDelayEx.argtypes = [c_uint32, c_uint32]
    api.ScpGetTriggerDelayMax.restype = c_double
    api.ScpGetTriggerDelayMax.argtypes = [c_uint32]
    api.ScpGetTriggerDelayMaxEx.restype = c_double
    api.ScpGetTriggerDelayMaxEx.argtypes = [c_uint32, c_uint32, c_double]
    api.ScpGetTriggerDelay.restype = c_double
    api.ScpGetTriggerDelay.argtypes = [c_uint32]
    api.ScpSetTriggerDelay.restype = c_double
    api.ScpSetTriggerDelay.argtypes = [c_uint32, c_double]
    api.ScpVerifyTriggerDelay.restype = c_double
    api.ScpVerifyTriggerDelay.argtypes = [c_uint32, c_double]
    api.ScpVerifyTriggerDelayEx.restype = c_double
    api.ScpVerifyTriggerDelayEx.argtypes = [c_uint32, c_double, c_uint32, c_double]
    api.ScpHasTriggerHoldOff.restype = c_uint8
    api.ScpHasTriggerHoldOff.argtypes = [c_uint32]
    api.ScpHasTriggerHoldOffEx.restype = c_uint8
    api.ScpHasTriggerHoldOffEx.argtypes = [c_uint32, c_uint32]
    api.ScpGetTriggerHoldOffCountMax.restype = c_uint64
    api.ScpGetTriggerHoldOffCountMax.argtypes = [c_uint32]
    api.ScpGetTriggerHoldOffCountMaxEx.restype = c_uint64
    api.ScpGetTriggerHoldOffCountMaxEx.argtypes = [c_uint32, c_uint32]
    api.ScpGetTriggerHoldOffCount.restype = c_uint64
    api.ScpGetTriggerHoldOffCount.argtypes = [c_uint32]
    api.ScpSetTriggerHoldOffCount.restype = c_uint64
    api.ScpSetTriggerHoldOffCount.argtypes = [c_uint32, c_uint64]
    api.ScpHasConnectionTest.restype = c_uint8
    api.ScpHasConnectionTest.argtypes = [c_uint32]
    api.ScpChHasConnectionTest.restype = c_uint8
    api.ScpChHasConnectionTest.argtypes = [c_uint32, c_uint16]
    api.ScpStartConnectionTest.restype = c_uint8
    api.ScpStartConnectionTest.argtypes = [c_uint32]
    api.ScpStartConnectionTestEx.restype = c_uint8
    api.ScpStartConnectionTestEx.argtypes = [c_uint32, c_void_p, c_uint16]
    api.ScpIsConnectionTestCompleted.restype = c_uint8
    api.ScpIsConnectionTestCompleted.argtypes = [c_uint32]
    api.ScpGetConnectionTestData.restype = c_uint16
    api.ScpGetConnectionTestData.argtypes = [c_uint32, c_void_p, c_uint16]
    api.GenGetConnectorType.restype = c_uint32
    api.GenGetConnectorType.argtypes = [c_uint32]
    api.GenIsDifferential.restype = c_uint8
    api.GenIsDifferential.argtypes = [c_uint32]
    api.GenGetImpedance.restype = c_double
    api.GenGetImpedance.argtypes = [c_uint32]
    api.GenGetResolution.restype = c_uint8
    api.GenGetResolution.argtypes = [c_uint32]
    api.GenGetOutputValueMin.restype = c_double
    api.GenGetOutputValueMin.argtypes = [c_uint32]
    api.GenGetOutputValueMax.restype = c_double
    api.GenGetOutputValueMax.argtypes = [c_uint32]
    api.GenGetOutputValueMinMax.restype = None
    api.GenGetOutputValueMinMax.argtypes = [c_uint32, c_void_p, c_void_p]
    api.GenIsControllable.restype = c_uint8
    api.GenIsControllable.argtypes = [c_uint32]
    api.GenIsRunning.restype = c_uint8
    api.GenIsRunning.argtypes = [c_uint32]
    api.GenGetStatus.restype = c_uint32
    api.GenGetStatus.argtypes = [c_uint32]
    api.GenGetOutputOn.restype = c_uint8
    api.GenGetOutputOn.argtypes = [c_uint32]
    api.GenSetOutputOn.restype = c_uint8
    api.GenSetOutputOn.argtypes = [c_uint32, c_uint8]
    api.GenHasOutputInvert.restype = c_uint8
    api.GenHasOutputInvert.argtypes = [c_uint32]
    api.GenGetOutputInvert.restype = c_uint8
    api.GenGetOutputInvert.argtypes = [c_uint32]
    api.GenSetOutputInvert.restype = c_uint8
    api.GenSetOutputInvert.argtypes = [c_uint32, c_uint8]
    api.GenStart.restype = c_uint8
    api.GenStart.argtypes = [c_uint32]
    api.GenStop.restype = c_uint8
    api.GenStop.argtypes = [c_uint32]
    api.GenGetSignalTypes.restype = c_uint32
    api.GenGetSignalTypes.argtypes = [c_uint32]
    api.GenGetSignalType.restype = c_uint32
    api.GenGetSignalType.argtypes = [c_uint32]
    api.GenSetSignalType.restype = c_uint32
    api.GenSetSignalType.argtypes = [c_uint32, c_uint32]
    api.GenHasAmplitude.restype = c_uint8
    api.GenHasAmplitude.argtypes = [c_uint32]
    api.GenHasAmplitudeEx.restype = c_uint8
    api.GenHasAmplitudeEx.argtypes = [c_uint32, c_uint32]
    api.GenGetAmplitudeMin.restype = c_double
    api.GenGetAmplitudeMin.argtypes = [c_uint32]
    api.GenGetAmplitudeMax.restype = c_double
    api.GenGetAmplitudeMax.argtypes = [c_uint32]
    api.GenGetAmplitudeMinMaxEx.restype = None
    api.GenGetAmplitudeMinMaxEx.argtypes = [c_uint32, c_uint32, c_void_p, c_void_p]
    api.GenGetAmplitude.restype = c_double
    api.GenGetAmplitude.argtypes = [c_uint32]
    api.GenSetAmplitude.restype = c_double
    api.GenSetAmplitude.argtypes = [c_uint32, c_double]
    api.GenVerifyAmplitude.restype = c_double
    api.GenVerifyAmplitude.argtypes = [c_uint32, c_double]
    api.GenVerifyAmplitudeEx.restype = c_double
    api.GenVerifyAmplitudeEx.argtypes = [c_uint32, c_double, c_uint32, c_uint32, c_double]
    api.GenGetAmplitudeRanges.restype = c_uint32
    api.GenGetAmplitudeRanges.argtypes = [c_uint32, c_void_p, c_uint32]
    api.GenGetAmplitudeRange.restype = c_double
    api.GenGetAmplitudeRange.argtypes = [c_uint32]
    api.GenSetAmplitudeRange.restype = c_double
    api.GenSetAmplitudeRange.argtypes = [c_uint32, c_double]
    api.GenGetAmplitudeAutoRanging.restype = c_uint8
    api.GenGetAmplitudeAutoRanging.argtypes = [c_uint32]
    api.GenSetAmplitudeAutoRanging.restype = c_uint8
    api.GenSetAmplitudeAutoRanging.argtypes = [c_uint32, c_uint8]
    api.GenHasOffset.restype = c_uint8
    api.GenHasOffset.argtypes = [c_uint32]
    api.GenHasOffsetEx.restype = c_uint8
    api.GenHasOffsetEx.argtypes = [c_uint32, c_uint32]
    api.GenGetOffsetMin.restype = c_double
    api.GenGetOffsetMin.argtypes = [c_uint32]
    api.GenGetOffsetMax.restype = c_double
    api.GenGetOffsetMax.argtypes = [c_uint32]
    api.GenGetOffsetMinMaxEx.restype = None
    api.GenGetOffsetMinMaxEx.argtypes = [c_uint32, c_uint32, c_void_p, c_void_p]
    api.GenGetOffset.restype = c_double
    api.GenGetOffset.argtypes = [c_uint32]
    api.GenSetOffset.restype = c_double
    api.GenSetOffset.argtypes = [c_uint32, c_double]
    api.GenVerifyOffset.restype = c_double
    api.GenVerifyOffset.argtypes = [c_uint32, c_double]
    api.GenVerifyOffsetEx.restype = c_double
    api.GenVerifyOffsetEx.argtypes = [c_uint32, c_double, c_uint32, c_double]
    api.GenGetFrequencyModes.restype = c_uint32
    api.GenGetFrequencyModes.argtypes = [c_uint32]
    api.GenGetFrequencyModesEx.restype = c_uint32
    api.GenGetFrequencyModesEx.argtypes = [c_uint32, c_uint32]
    api.GenGetFrequencyMode.restype = c_uint32
    api.GenGetFrequencyMode.argtypes = [c_uint32]
    api.GenSetFrequencyMode.restype = c_uint32
    api.GenSetFrequencyMode.argtypes = [c_uint32, c_uint32]
    api.GenHasFrequency.restype = c_uint8
    api.GenHasFrequency.argtypes = [c_uint32]
    api.GenHasFrequencyEx.restype = c_uint8
    api.GenHasFrequencyEx.argtypes = [c_uint32, c_uint32, c_uint32]
    api.GenGetFrequencyMin.restype = c_double
    api.GenGetFrequencyMin.argtypes = [c_uint32]
    api.GenGetFrequencyMax.restype = c_double
    api.GenGetFrequencyMax.argtypes = [c_uint32]
    api.GenGetFrequencyMinMax.restype = None
    api.GenGetFrequencyMinMax.argtypes = [c_uint32, c_uint32, c_void_p, c_void_p]
    api.GenGetFrequencyMinMaxEx.restype = None
    api.GenGetFrequencyMinMaxEx.argtypes = [c_uint32, c_uint32, c_uint32, c_void_p, c_void_p]
    api.GenGetFrequency.restype = c_double
    api.GenGetFrequency.argtypes = [c_uint32]
    api.GenSetFrequency.restype = c_double
    api.GenSetFrequency.argtypes = [c_uint32, c_double]
    api.GenVerifyFrequency.restype = c_double
    api.GenVerifyFrequency.argtypes = [c_uint32, c_double]
    api.GenVerifyFrequencyEx2.restype = c_double
    api.GenVerifyFrequencyEx2.argtypes = [c_uint32, c_double, c_uint32, c_uint32, c_uint64, c_double]
    api.GenHasPhase.restype = c_uint8
    api.GenHasPhase.argtypes = [c_uint32]
    api.GenHasPhaseEx.restype = c_uint8
    api.GenHasPhaseEx.argtypes = [c_uint32, c_uint32]
    api.GenGetPhaseMin.restype = c_double
    api.GenGetPhaseMin.argtypes = [c_uint32]
    api.GenGetPhaseMax.restype = c_double
    api.GenGetPhaseMax.argtypes = [c_uint32]
    api.GenGetPhaseMinMaxEx.restype = None
    api.GenGetPhaseMinMaxEx.argtypes = [c_uint32, c_uint32, c_void_p, c_void_p]
    api.GenGetPhase.restype = c_double
    api.GenGetPhase.argtypes = [c_uint32]
    api.GenSetPhase.restype = c_double
    api.GenSetPhase.argtypes = [c_uint32, c_double]
    api.GenVerifyPhase.restype = c_double
    api.GenVerifyPhase.argtypes = [c_uint32, c_double]
    api.GenVerifyPhaseEx.restype = c_double
    api.GenVerifyPhaseEx.argtypes = [c_uint32, c_double, c_uint32]
    api.GenHasSymmetry.restype = c_uint8
    api.GenHasSymmetry.argtypes = [c_uint32]
    api.GenHasSymmetryEx.restype = c_uint8
    api.GenHasSymmetryEx.argtypes = [c_uint32, c_uint32]
    api.GenGetSymmetryMin.restype = c_double
    api.GenGetSymmetryMin.argtypes = [c_uint32]
    api.GenGetSymmetryMax.restype = c_double
    api.GenGetSymmetryMax.argtypes = [c_uint32]
    api.GenGetSymmetryMinMaxEx.restype = None
    api.GenGetSymmetryMinMaxEx.argtypes = [c_uint32, c_uint32, c_void_p, c_void_p]
    api.GenGetSymmetry.restype = c_double
    api.GenGetSymmetry.argtypes = [c_uint32]
    api.GenSetSymmetry.restype = c_double
    api.GenSetSymmetry.argtypes = [c_uint32, c_double]
    api.GenVerifySymmetry.restype = c_double
    api.GenVerifySymmetry.argtypes = [c_uint32, c_double]
    api.GenVerifySymmetryEx.restype = c_double
    api.GenVerifySymmetryEx.argtypes = [c_uint32, c_double, c_uint32]
    api.GenHasWidth.restype = c_uint8
    api.GenHasWidth.argtypes = [c_uint32]
    api.GenHasWidthEx.restype = c_uint8
    api.GenHasWidthEx.argtypes = [c_uint32, c_uint32]
    api.GenGetWidthMin.restype = c_double
    api.GenGetWidthMin.argtypes = [c_uint32]
    api.GenGetWidthMax.restype = c_double
    api.GenGetWidthMax.argtypes = [c_uint32]
    api.GenGetWidthMinMaxEx.restype = None
    api.GenGetWidthMinMaxEx.argtypes = [c_uint32, c_uint32, c_double, c_void_p, c_void_p]
    api.GenGetWidth.restype = c_double
    api.GenGetWidth.argtypes = [c_uint32]
    api.GenSetWidth.restype = c_double
    api.GenSetWidth.argtypes = [c_uint32, c_double]
    api.GenVerifyWidth.restype = c_double
    api.GenVerifyWidth.argtypes = [c_uint32, c_double]
    api.GenVerifyWidthEx.restype = c_double
    api.GenVerifyWidthEx.argtypes = [c_uint32, c_double, c_uint32, c_double]
    api.GenHasEdgeTime.restype = c_uint8
    api.GenHasEdgeTime.argtypes = [c_uint32]
    api.GenHasEdgeTimeEx.restype = c_uint8
    api.GenHasEdgeTimeEx.argtypes = [c_uint32, c_uint32]
    api.GenGetLeadingEdgeTimeMin.restype = c_double
    api.GenGetLeadingEdgeTimeMin.argtypes = [c_uint32]
    api.GenGetLeadingEdgeTimeMax.restype = c_double
    api.GenGetLeadingEdgeTimeMax.argtypes = [c_uint32]
    api.GenGetLeadingEdgeTimeMinMaxEx.restype = None
    api.GenGetLeadingEdgeTimeMinMaxEx.argtypes = [c_uint32, c_uint32, c_double, c_double, c_double, c_double, c_void_p, c_void_p]
    api.GenGetLeadingEdgeTime.restype = c_double
    api.GenGetLeadingEdgeTime.argtypes = [c_uint32]
    api.GenSetLeadingEdgeTime.restype = c_double
    api.GenSetLeadingEdgeTime.argtypes = [c_uint32, c_double]
    api.GenVerifyLeadingEdgeTime.restype = c_double
    api.GenVerifyLeadingEdgeTime.argtypes = [c_uint32, c_double]
    api.GenVerifyLeadingEdgeTimeEx.restype = c_double
    api.GenVerifyLeadingEdgeTimeEx.argtypes = [c_uint32, c_double, c_uint32, c_double, c_double, c_double, c_double]
    api.GenGetTrailingEdgeTimeMin.restype = c_double
    api.GenGetTrailingEdgeTimeMin.argtypes = [c_uint32]
    api.GenGetTrailingEdgeTimeMax.restype = c_double
    api.GenGetTrailingEdgeTimeMax.argtypes = [c_uint32]
    api.GenGetTrailingEdgeTimeMinMaxEx.restype = None
    api.GenGetTrailingEdgeTimeMinMaxEx.argtypes = [c_uint32, c_uint32, c_double, c_double, c_double, c_double, c_void_p, c_void_p]
    api.GenGetTrailingEdgeTime.restype = c_double
    api.GenGetTrailingEdgeTime.argtypes = [c_uint32]
    api.GenSetTrailingEdgeTime.restype = c_double
    api.GenSetTrailingEdgeTime.argtypes = [c_uint32, c_double]
    api.GenVerifyTrailingEdgeTime.restype = c_double
    api.GenVerifyTrailingEdgeTime.argtypes = [c_uint32, c_double]
    api.GenVerifyTrailingEdgeTimeEx.restype = c_double
    api.GenVerifyTrailingEdgeTimeEx.argtypes = [c_uint32, c_double, c_uint32, c_double, c_double, c_double, c_double]
    api.GenHasData.restype = c_uint8
    api.GenHasData.argtypes = [c_uint32]
    api.GenHasDataEx.restype = c_uint8
    api.GenHasDataEx.argtypes = [c_uint32, c_uint32]
    api.GenGetDataLengthMin.restype = c_uint64
    api.GenGetDataLengthMin.argtypes = [c_uint32]
    api.GenGetDataLengthMax.restype = c_uint64
    api.GenGetDataLengthMax.argtypes = [c_uint32]
    api.GenGetDataLengthMinMaxEx.restype = None
    api.GenGetDataLengthMinMaxEx.argtypes = [c_uint32, c_uint32, c_void_p, c_void_p]
    api.GenGetDataLength.restype = c_uint64
    api.GenGetDataLength.argtypes = [c_uint32]
    api.GenVerifyDataLength.restype = c_uint64
    api.GenVerifyDataLength.argtypes = [c_uint32, c_uint64]
    api.GenVerifyDataLengthEx.restype = c_uint64
    api.GenVerifyDataLengthEx.argtypes = [c_uint32, c_uint64, c_uint32]
    api.GenSetData.restype = None
    api.GenSetData.argtypes = [c_uint32, c_void_p, c_uint64]
    api.GenSetDataEx.restype = None
    api.GenSetDataEx.argtypes = [c_uint32, c_void_p, c_uint64, c_uint32, c_uint32]
    api.GenGetDataRawType.restype = c_uint32
    api.GenGetDataRawType.argtypes = [c_uint32]
    api.GenGetDataRawValueRange.restype = None
    api.GenGetDataRawValueRange.argtypes = [c_uint32, c_void_p, c_void_p, c_void_p]
    api.GenGetDataRawValueMin.restype = c_int64
    api.GenGetDataRawValueMin.argtypes = [c_uint32]
    api.GenGetDataRawValueZero.restype = c_int64
    api.GenGetDataRawValueZero.argtypes = [c_uint32]
    api.GenGetDataRawValueMax.restype = c_int64
    api.GenGetDataRawValueMax.argtypes = [c_uint32]
    api.GenSetDataRaw.restype = None
    api.GenSetDataRaw.argtypes = [c_uint32, c_void_p, c_uint64]
    api.GenSetDataRawEx.restype = None
    api.GenSetDataRawEx.argtypes = [c_uint32, c_void_p, c_uint64, c_uint32, c_uint32]
    api.GenGetModes.restype = c_uint64
    api.GenGetModes.argtypes = [c_uint32]
    api.GenGetModesEx.restype = c_uint64
    api.GenGetModesEx.argtypes = [c_uint32, c_uint32, c_uint32]
    api.GenGetModesNative.restype = c_uint64
    api.GenGetModesNative.argtypes = [c_uint32]
    api.GenGetMode.restype = c_uint64
    api.GenGetMode.argtypes = [c_uint32]
    api.GenSetMode.restype = c_uint64
    api.GenSetMode.argtypes = [c_uint32, c_uint64]
    api.GenIsBurstActive.restype = c_uint8
    api.GenIsBurstActive.argtypes = [c_uint32]
    api.GenGetBurstCountMin.restype = c_uint64
    api.GenGetBurstCountMin.argtypes = [c_uint32]
    api.GenGetBurstCountMax.restype = c_uint64
    api.GenGetBurstCountMax.argtypes = [c_uint32]
    api.GenGetBurstCountMinMaxEx.restype = None
    api.GenGetBurstCountMinMaxEx.argtypes = [c_uint32, c_uint64, c_void_p, c_void_p]
    api.GenGetBurstCount.restype = c_uint64
    api.GenGetBurstCount.argtypes = [c_uint32]
    api.GenSetBurstCount.restype = c_uint64
    api.GenSetBurstCount.argtypes = [c_uint32, c_uint64]
    api.GenGetBurstSampleCountMin.restype = c_uint64
    api.GenGetBurstSampleCountMin.argtypes = [c_uint32]
    api.GenGetBurstSampleCountMax.restype = c_uint64
    api.GenGetBurstSampleCountMax.argtypes = [c_uint32]
    api.GenGetBurstSampleCountMinMaxEx.restype = None
    api.GenGetBurstSampleCountMinMaxEx.argtypes = [c_uint32, c_uint64, c_void_p, c_void_p]
    api.GenGetBurstSampleCount.restype = c_uint64
    api.GenGetBurstSampleCount.argtypes = [c_uint32]
    api.GenSetBurstSampleCount.restype = c_uint64
    api.GenSetBurstSampleCount.argtypes = [c_uint32, c_uint64]
    api.GenGetBurstSegmentCountMin.restype = c_uint64
    api.GenGetBurstSegmentCountMin.argtypes = [c_uint32]
    api.GenGetBurstSegmentCountMax.restype = c_uint64
    api.GenGetBurstSegmentCountMax.argtypes = [c_uint32]
    api.GenGetBurstSegmentCountMinMaxEx.restype = None
    api.GenGetBurstSegmentCountMinMaxEx.argtypes = [c_uint32, c_uint64, c_uint32, c_uint32, c_double, c_uint64, c_void_p, c_void_p]
    api.GenGetBurstSegmentCount.restype = c_uint64
    api.GenGetBurstSegmentCount.argtypes = [c_uint32]
    api.GenSetBurstSegmentCount.restype = c_uint64
    api.GenSetBurstSegmentCount.argtypes = [c_uint32, c_uint64]
    api.GenVerifyBurstSegmentCount.restype = c_uint64
    api.GenVerifyBurstSegmentCount.argtypes = [c_uint32, c_uint64]
    api.GenVerifyBurstSegmentCountEx.restype = c_uint64
    api.GenVerifyBurstSegmentCountEx.argtypes = [c_uint32, c_uint64, c_uint64, c_uint32, c_uint32, c_double, c_uint64]
    api.GenSetCallbackBurstCompleted.restype = None
    api.GenSetCallbackBurstCompleted.argtypes = [c_uint32, Callback, c_void_p]
    if platform.system() == 'Linux':
        api.GenSetEventBurstCompleted.restype = None
        api.GenSetEventBurstCompleted.argtypes = [c_uint32, c_int]
    if platform.system() == 'Windows':
        api.GenSetEventBurstCompleted.restype = None
        api.GenSetEventBurstCompleted.argtypes = [c_uint32, HANDLE]
        api.GenSetMessageBurstCompleted.restype = None
        api.GenSetMessageBurstCompleted.argtypes = [c_uint32, HWND, WPARAM, LPARAM]
    api.GenSetCallbackControllableChanged.restype = None
    api.GenSetCallbackControllableChanged.argtypes = [c_uint32, Callback, c_void_p]
    if platform.system() == 'Linux':
        api.GenSetEventControllableChanged.restype = None
        api.GenSetEventControllableChanged.argtypes = [c_uint32, c_int]
    if platform.system() == 'Windows':
        api.GenSetEventControllableChanged.restype = None
        api.GenSetEventControllableChanged.argtypes = [c_uint32, HANDLE]
        api.GenSetMessageControllableChanged.restype = None
        api.GenSetMessageControllableChanged.argtypes = [c_uint32, HWND, WPARAM, LPARAM]
    api.I2CIsInternalAddress.restype = c_uint8
    api.I2CIsInternalAddress.argtypes = [c_uint32, c_uint16]
    api.I2CGetInternalAddresses.restype = c_uint32
    api.I2CGetInternalAddresses.argtypes = [c_uint32, c_void_p, c_uint32]
    api.I2CRead.restype = c_uint8
    api.I2CRead.argtypes = [c_uint32, c_uint16, c_void_p, c_uint32, c_uint8]
    api.I2CReadByte.restype = c_uint8
    api.I2CReadByte.argtypes = [c_uint32, c_uint16, c_void_p]
    api.I2CReadWord.restype = c_uint8
    api.I2CReadWord.argtypes = [c_uint32, c_uint16, c_void_p]
    api.I2CWrite.restype = c_uint8
    api.I2CWrite.argtypes = [c_uint32, c_uint16, c_void_p, c_uint32, c_uint8]
    api.I2CWriteByte.restype = c_uint8
    api.I2CWriteByte.argtypes = [c_uint32, c_uint16, c_uint8]
    api.I2CWriteByteByte.restype = c_uint8
    api.I2CWriteByteByte.argtypes = [c_uint32, c_uint16, c_uint8, c_uint8]
    api.I2CWriteByteWord.restype = c_uint8
    api.I2CWriteByteWord.argtypes = [c_uint32, c_uint16, c_uint8, c_uint16]
    api.I2CWriteWord.restype = c_uint8
    api.I2CWriteWord.argtypes = [c_uint32, c_uint16, c_uint16]
    api.I2CWriteRead.restype = c_uint8
    api.I2CWriteRead.argtypes = [c_uint32, c_uint16, c_void_p, c_uint32, c_void_p, c_uint32]
    api.I2CGetSpeedMax.restype = c_double
    api.I2CGetSpeedMax.argtypes = [c_uint32]
    api.I2CGetSpeed.restype = c_double
    api.I2CGetSpeed.argtypes = [c_uint32]
    api.I2CSetSpeed.restype = c_double
    api.I2CSetSpeed.argtypes = [c_uint32, c_double]
    api.I2CVerifySpeed.restype = c_double
    api.I2CVerifySpeed.argtypes = [c_uint32, c_double]
    api.SrvConnect.restype = c_uint8
    api.SrvConnect.argtypes = [c_uint32, c_uint8]
    api.SrvDisconnect.restype = c_uint8
    api.SrvDisconnect.argtypes = [c_uint32, c_uint8]
    api.SrvRemove.restype = c_uint8
    api.SrvRemove.argtypes = [c_uint32, c_uint8]
    api.SrvGetStatus.restype = c_uint32
    api.SrvGetStatus.argtypes = [c_uint32]
    api.SrvGetLastError.restype = c_uint32
    api.SrvGetLastError.argtypes = [c_uint32]
    api.SrvGetURL.restype = c_uint32
    api.SrvGetURL.argtypes = [c_uint32, c_char_p, c_uint32]
    api.SrvGetID.restype = c_uint32
    api.SrvGetID.argtypes = [c_uint32, c_char_p, c_uint32]
    api.SrvGetIPv4Address.restype = c_uint32
    api.SrvGetIPv4Address.argtypes = [c_uint32]
    api.SrvGetIPPort.restype = c_uint16
    api.SrvGetIPPort.argtypes = [c_uint32]
    api.SrvGetName.restype = c_uint32
    api.SrvGetName.argtypes = [c_uint32, c_char_p, c_uint32]
    api.SrvGetDescription.restype = c_uint32
    api.SrvGetDescription.argtypes = [c_uint32, c_char_p, c_uint32]
    api.SrvGetVersion.restype = c_uint64
    api.SrvGetVersion.argtypes = [c_uint32]
    api.SrvGetVersionExtra.restype = c_uint32
    api.SrvGetVersionExtra.argtypes = [c_uint32, c_char_p, c_uint32]
    api.HlpPointerArrayNew.restype = c_void_p
    api.HlpPointerArrayNew.argtypes = [c_uint32]
    api.HlpPointerArraySet.restype = None
    api.HlpPointerArraySet.argtypes = [c_void_p, c_uint32, c_void_p]
    api.HlpPointerArrayDelete.restype = None
    api.HlpPointerArrayDelete.argtypes = [c_void_p]

    return api


if 'sphinx' in sys.modules:
    class API:
        def LibInit(self):
            pass

        def LibExit(self):
            pass

    api = API()
else:
    api = load_libtiepie()
